SystemOrganization addCategory: #GoogleStaticMap!
SystemOrganization addCategory: 'GoogleStaticMap-Examples'!

!WAHtmlCanvas methodsFor: '*GoogleStaticMap' stamp: 'rs 12/26/2008 09:55'!
googleStaticMap

	^self brush: GMStaticMap new! !

WAImageTag subclass: #GMStaticMap
	instanceVariableNames: 'longitude latitude zoom mapType scale path imageFormat markerGroups styles language'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMStaticMap commentStamp: '<historical>' prior: 0!
GMStaticMap is the integration of static maps with seaside - it is a subclass of the image tag that renders th src-attribute according to the API of Google static maps.

Instance Variables:
	longitude		<Float>
	latitude			<Float>
	type			<String>
	zoom			<Integer>
	mobile			<Boolean>
	path			<GMStaticMapPath>
	imageFormat	<String>
	markerGroups	<Collection of GMStaticMapPointGroup!

!GMStaticMap class methodsFor: 'constants' stamp: 'rs 3/29/2010 16:01'!
supportedImageFormats
	"Return a list of all image formats allowed for static maps" 

	^#('png' 'png8' 'png32' 'gif' 'jpg' 'jpg-baseline')
! !

!GMStaticMap class methodsFor: 'constants' stamp: 'RichardStern 4/17/2014 14:35'!
supportedLanguages
	"Return a list of all languages allowed for static maps" 

	^#('ar' 'bg' 'bn' 'ca' 'cs' 'da' 'de' 'el' 'en' 'es' 'eu' 'fi' 'fil' 'fr' 'gl' 'gu' 'hi' 'hr' 'hu' 'id' 'it' 'iw' 'ja' 'kn' 'ko' 'lt' 'lv' 'ml' 'mr' 'nl' 'nn' 'no' 'or' 'pl' 'pt' 'pt-BR' 'pt-PT' 'rm' 'ro' 'ru' 'sk' 'sl' 'sr' 'sv' 'ta' 'te' 'th' 'tr' 'uk' 'vi' 'zh-CN' 'zh-TW' )
! !

!GMStaticMap class methodsFor: 'constants' stamp: 'RichardStern 12/6/2010 11:22'!
supportedMapTypes
	"Return a list of all map type values allowed for static maps" 

	^#('roadmap' 'satellite' 'terrain' 'hybrid')
! !

!GMStaticMap class methodsFor: 'constants' stamp: 'rs 7/29/2013 08:14'!
supportedScaleValues
	"Return a list of all scale values allowed for static maps" 

	^#(1 2 4)
! !

!GMStaticMap methodsFor: 'public' stamp: 'RichardStern 12/31/2010 12:48'!
addMarkerGroup: aGMStaticMapMarkerGroup
	"Add passed marker group to receiver"
	
	self markerGroups add: aGMStaticMapMarkerGroup.! !

!GMStaticMap methodsFor: 'public' stamp: 'rs 12/11/2013 12:54'!
addMarkerGroups: aCollOfMarkerGroups
	"Add all passed marker groups to receiver"

	aCollOfMarkerGroups do: [:m| self addMarkerGroup: m.].! !

!GMStaticMap methodsFor: 'private-url assembling' stamp: 'rs 7/25/2013 08:57'!
addMarkersToUrl: aWAUrl
	"Add markers to the passed map url"
	"In difference to former API versions, equal looking markers are
	now stored as groups - add each group to URL here"	

	self hasMarkers ifTrue:
	[
		self markerGroups do: 
		[:m | | stream |	
			stream := WriteStream on: String new.
			m asUrlParameterOn: stream.
			aWAUrl addField: 'markers' value: stream contents.
		].
	].
! !

!GMStaticMap methodsFor: 'private-url assembling' stamp: 'rs 7/25/2013 09:01'!
addPositionAndExtentParametersToUrl: aWAUrl
	"To the URL, add all the different config parameters controlling the position 
	and size of the map"

	self hasCenter ifTrue: [ aWAUrl addField: 'center' value: self mapCenter.].	
	self hasSize ifTrue: [ aWAUrl addField: 'size' value: self mapSize.].
	self hasScale ifTrue: [aWAUrl addField: 'scale' value: self scale.].
	self hasZoom ifTrue: [ aWAUrl addField: 'zoom' value: self zoom.].		
			
! !

!GMStaticMap methodsFor: 'public' stamp: 'rs 12/11/2013 12:53'!
addStyle:  aGMStaticMapStyle
	"Add passed style definition to receiver"
	
	self styles add: aGMStaticMapStyle.! !

!GMStaticMap methodsFor: 'public' stamp: 'rs 12/11/2013 12:54'!
addStyles:  aCollOfGMStaticMapStyles
	"Add passed style definitions to receiver"
	
	aCollOfGMStaticMapStyles do: [:s | self addStyle: s.].! !

!GMStaticMap methodsFor: 'private-url assembling' stamp: 'rs 12/11/2013 12:59'!
addStylesToUrl: aWAUrl
	"Add styles to the passed map url"

	self hasStyles ifTrue:
	[	| stream |	
		stream := WriteStream on: String new.
		self styles do: 
		[:s | 
			s asUrlParameterOn: stream.
		].
		aWAUrl addField: 'style' value: stream contents.
	].
! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'RichardStern 12/31/2010 12:55'!
hasCenter
	"Return whether map has center defined, ie, both longitude and latitude value are set"
	
	^(self longitude~~nil) and: [self latitude~~nil].! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'RichardStern 12/31/2010 12:55'!
hasImageFormat
	"Return whether target format for imag ehas been set"
	
	^(self imageFormat~~nil).! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'RichardStern 4/17/2014 14:24'!
hasLanguage
	"Return wheter a language was quoted for receiver"
	
	^(self language~~nil).! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'rs 7/25/2013 08:52'!
hasMapType
	"Return wheter a map type was quoted for receiver"
	
	^(self mapType~~nil).! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'rs 12/11/2013 13:12'!
hasMarkers
	"Return whether receiver involves any markers"
	
	^self markerGroups isCollection and: [self markerGroups isEmpty not].! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'RichardStern 12/31/2010 12:55'!
hasPath
	"Return whether receiver show a path (ie, a polyline) on the map"
	
	^(self path~~nil)! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'rs 7/25/2013 08:24'!
hasScale
	"Return wheter a scale was quoted for receiver"
	
	^(self scale~~nil).! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'RichardStern 12/31/2010 12:56'!
hasSize
	"Return whether map has size defined, ie, both width and height value are set"
		
	^(self width~~nil) and: [self height~~nil].! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'rs 12/11/2013 13:12'!
hasStyles
	"Return whether receiver involves any styles"
	
	^self styles isCollection and: [self styles isEmpty not].! !

!GMStaticMap methodsFor: 'private-testing' stamp: 'RichardStern 12/31/2010 12:56'!
hasZoom
	"Return wheter a zoom level was quoted for receiver"
	
	^(self zoom~~nil).! !

!GMStaticMap methodsFor: 'private-image attributes' stamp: 'rs 7/25/2013 08:49'!
height
	"Return the height of the map - which is a property inherited from image tag"
	"Means that map is always as big as the image tag itself - though they could have
	different extents in theory. So not fully consitent, but same size is usually the intention"	
	
	^self attributes at: 'height' ifAbsent: [nil].
! !

!GMStaticMap methodsFor: 'public' stamp: 'rs 7/25/2013 08:38'!
hybridMap 
	"Set hybrid (satellite+map) display mode for the map"
	
	self mapType: 'hybrid'! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 3/29/2010 16:00'!
imageFormat
	^ imageFormat! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 3/29/2010 16:00'!
imageFormat: anObject
	imageFormat := anObject! !

!GMStaticMap methodsFor: 'private-url assembling' stamp: 'RichardStern 9/21/2015 13:08'!
imageUrl
	"Assemble the url-attribute of the image tag. This is the central method responsible for actually 
	getting an url according to Google static maps API that finally gets the map image "
	
	| url |

	"Get the basic URL for the API"
	url := WAUrl new
			scheme: 'http';
			host: 'maps.google.com';
			addToPath: 'maps/api/staticmap';
			yourself.

	"To the URL, add all the different config parameters (if defined)"
	self addPositionAndExtentParametersToUrl: url.

	"Add url parameters controlling image format and map type"
	self hasImageFormat
		ifTrue: [ url addField: 'format' value: self imageFormat.].	
	self hasMapType
		ifTrue: [ url addField: 'maptype' value: self mapType.].	

	"Add language setting"
	self hasLanguage
		ifTrue: [ url addField: 'language' value: self language.].				

	"Add sensor parameter. This is mandatory for all map request and therefore
	assumed false by default. ( GPS sensor support not considered yet)"
	 url addField: 'sensor' value: 'false'.
			
	"Add any markers and paths defined for receiver"
	self addMarkersToUrl: url.
	self hasPath ifTrue: [ self path addYourselfToUrl: url.].	
		
	"Add styles"
	self addStylesToUrl: url.

	^url! !

!GMStaticMap methodsFor: 'initializing' stamp: 'RichardStern 4/17/2014 14:27'!
initialize
	super initialize.
	
	markerGroups := OrderedCollection new.
	styles := OrderedCollection new.

! !

!GMStaticMap methodsFor: 'accessing' stamp: 'RichardStern 4/17/2014 14:24'!
language
	^ language! !

!GMStaticMap methodsFor: 'accessing' stamp: 'RichardStern 4/17/2014 14:24'!
language: anObject
	language := anObject! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 12/23/2008 15:17'!
latitude
	^ latitude! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 12/23/2008 15:17'!
latitude: anObject
	latitude := anObject! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 12/23/2008 15:17'!
longitude
	^ longitude! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 12/23/2008 15:17'!
longitude: anObject
	longitude := anObject! !

!GMStaticMap methodsFor: 'private-map attributes' stamp: 'rs 7/25/2013 08:51'!
mapCenter
	"Return the center of the map expressed in geocoordinate string - e.g. '16.64,14.88'"
	
	^self latitude asString,',',self longitude asString.! !

!GMStaticMap methodsFor: 'private-map attributes' stamp: 'RichardStern 12/31/2010 12:53'!
mapSize
	"Return map size - which is a string liek e.g. '600x400''" 
	
	^(self attributes at: 'width') asString,'x',((self attributes at: 'height') asString).! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 7/25/2013 08:50'!
mapType
	^ mapType! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 7/25/2013 08:50'!
mapType: anObject
	mapType := anObject! !

!GMStaticMap methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 14:30'!
markerGroups
	^ markerGroups! !

!GMStaticMap methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 14:30'!
markerGroups: aCollection
	markerGroups := aCollection! !

!GMStaticMap methodsFor: 'private-url assembling' stamp: 'rs 7/25/2013 08:56'!
openTag
	"Override superclass to customize the tag output in order to put
	a target url that retrieves the map"
	
	self url: self imageUrl.
	super openTag! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 1/12/2010 14:58'!
path
	^ path! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 1/12/2010 14:58'!
path: anObject
	path := anObject! !

!GMStaticMap methodsFor: 'public' stamp: 'rs 7/25/2013 08:38'!
roadMap
	"Configure map to be shown as road map"
	
	self mapType: 'roadmap'! !

!GMStaticMap methodsFor: 'public' stamp: 'rs 7/25/2013 08:38'!
satelliteMap
	"Configure map to be shown as satellite image"
	
	self mapType: 'satellite'! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 7/25/2013 08:50'!
scale
	^ scale! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 7/25/2013 08:50'!
scale: anObject
	scale := anObject! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 12/11/2013 12:54'!
styles
	^ styles! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 12/11/2013 12:54'!
styles: aCollection
	styles := aCollection! !

!GMStaticMap methodsFor: 'public' stamp: 'rs 7/25/2013 08:38'!
terrainMap
	"Configure map to be shown with terrain info"
	
	self mapType: 'terrain'! !

!GMStaticMap methodsFor: 'public' stamp: 'RichardStern 12/31/2010 12:50'!
useMobileDeviceMap
	"Configure receiver to show map optimized for mobile output devices"
	
	self mobile: true.! !

!GMStaticMap methodsFor: 'private-image attributes' stamp: 'rs 7/25/2013 08:48'!
width
	"Return the width of the map - which is a property inherited from image tag"
	"Means that map is always as big as the image tag itself - though they could have
	different extents in theory. So not fully consitent, but same size is usually the intention"	

	^self attributes at: 'width' ifAbsent: [nil].
! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 12/23/2008 16:19'!
zoom
	^ zoom! !

!GMStaticMap methodsFor: 'accessing' stamp: 'rs 12/23/2008 16:19'!
zoom: anObject
	zoom := anObject! !

WAComponent subclass: #GMStaticMapExampleBase
	instanceVariableNames: 'longitude latitude width height scale zoom imageFormat markers mapType styles language'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

GMStaticMapExampleBase subclass: #GMStaticMapExampleAddressSearch
	instanceVariableNames: 'address outputFormat searchResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

!GMStaticMapExampleAddressSearch methodsFor: 'accessing' stamp: 'rs 1/21/2010 13:02'!
address
	^ address! !

!GMStaticMapExampleAddressSearch methodsFor: 'accessing' stamp: 'rs 1/21/2010 13:02'!
address: anObject
	address := anObject! !

!GMStaticMapExampleAddressSearch methodsFor: 'accessing' stamp: 'rs 3/30/2010 09:26'!
outputFormat
	^ outputFormat! !

!GMStaticMapExampleAddressSearch methodsFor: 'accessing' stamp: 'rs 3/30/2010 09:26'!
outputFormat: anObject
	outputFormat := anObject! !

!GMStaticMapExampleAddressSearch methodsFor: 'rendering' stamp: 'rs 3/30/2010 09:38'!
renderAddressFormOn: html

	html heading
			level: 5;
			with: 'Address search'.

	html form:
	[
		html table:
		[
			self renderInputCaptionsOn: html.
			
			self 
				renderInputFor: #address
				label: 'Address' 
				hint: 'Search string'
				model: self
				on: html.
	

			self renderOutputFormatSelectionOn: html.
		].		

		html submitButton 
				value: 'Search';
				callback: [self searchAddress].				

	].
		
! !

!GMStaticMapExampleAddressSearch methodsFor: 'rendering' stamp: 'rs 12/11/2013 08:29'!
renderDescriptionOn: html

	html 
		text: 'Maps can also be gained by simply quoting an address to search for; the result is got as json or xml and then parsed to assmble a map. For multiple results, the map adjust to show all of them . You can try this e.g. by searching a mere city name like '.
	html anchor
		callback: [self address: 'Vienna'; searchAddress.];
		with: 'Vienna'.
	html text: '.'.! !

!GMStaticMapExampleAddressSearch methodsFor: 'rendering' stamp: 'rs 8/9/2013 08:58'!
renderFormsOn: html

	self renderAddressFormOn: html.
	html break.
	self renderSearchResultMarkersOn: html.
	html break.
	self renderSearchResultOn: html.
! !

!GMStaticMapExampleAddressSearch methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 07:52'!
renderOutputFormatSelectionOn: html
	"Render dropdown box allowing to choose output format of address search"
			
	self
		renderDropdownFor: #outputFormat 
		label: 'Output format' 
		hint: '-' 
		model: self
		withValues: (GMGeocodeRequest supportedOutputFormats) 
		on: html.	
			
! !

!GMStaticMapExampleAddressSearch methodsFor: 'rendering' stamp: 'RichardStern 12/6/2010 14:57'!
renderResultLineFor: aGMStaticMapMarker on: html

	html tableRow:
	[
		html 
			tableData: aGMStaticMapMarker label asString;
			tableData: aGMStaticMapMarker points first longitude asString;
			tableData: aGMStaticMapMarker points first latitude asString
	].! !

!GMStaticMapExampleAddressSearch methodsFor: 'rendering' stamp: 'rs 3/30/2010 09:34'!
renderSearchResultMarkersOn: html
	"Search result is given in form a markers for sake of simplcity. Render them here if any present"

	self markers isEmpty ifFalse:
	[
		html heading
				level: 5;
				with: 'Found locations'.
				
		html table:
		[
			self markers do:
			[:m |
				self renderResultLineFor: m on: html.
			].
		].
	]! !

!GMStaticMapExampleAddressSearch methodsFor: 'rendering' stamp: 'rs 8/9/2013 09:48'!
renderSearchResultOn: html
	"Render the raw serach result in its original format (as it comes from service)"

	self searchResult isEmpty ifFalse:
	[
		html heading
				level: 5;
				with: 'Raw search result'.

		html div 
			class: 'searchResult';
			with:
			[
				html teletype: self searchResult.
			].
	].! !

!GMStaticMapExampleAddressSearch methodsFor: 'actions' stamp: 'RichardStern 4/17/2014 09:16'!
searchAddress
	| request |

	"Prepare the geocode request"
	request := GMGeocodeRequest new
						address: self address;
						outputFormat: self outputFormat;
						yourself.
									
	"Get search result as text for output in example"
	self searchResult: request	resultString.	

	"Now get markers from result (Note that request object cached the result)"
	self removeAllMarkers.
	self markers addAll: request resultMarkers.

	self markers isEmpty ifFalse:
	[	
		self centerAndZoomBasedOnMarkers.
	].! !

!GMStaticMapExampleAddressSearch methodsFor: 'accessing' stamp: 'rs 3/30/2010 09:26'!
searchResult
	^ searchResult! !

!GMStaticMapExampleAddressSearch methodsFor: 'accessing' stamp: 'rs 3/30/2010 09:26'!
searchResult: anObject
	searchResult := anObject! !

!GMStaticMapExampleAddressSearch methodsFor: 'private' stamp: 'RichardStern 4/17/2014 08:19'!
setInitialParameters
	
	super setInitialParameters.

	self 
		address: String new;
		searchResult: String new;
		outputFormat: 'json';
		zoom: '16'.! !

!GMStaticMapExampleAddressSearch methodsFor: 'rendering' stamp: 'rs 8/9/2013 09:49'!
style
	^super style,
	'
	div.searchResult 
	{	border: 1px solid #c0c0c0;
		margin-top: 1em;
		overflow: auto;
		width: 25em;
		height: 10em;
		font-size: 7pt;
	}

	'! !

!GMStaticMapExampleBase methodsFor: 'private' stamp: 'rs 8/9/2013 09:40'!
centerAndZoomBasedOnMarkers
	"Assume that markers are (or will be) added to the map and that cetner/zoom should be set to
	make all of them visible on the map. Do this by simply setting cetner and zoom values to nil -
	API that will use marker positions automatically"
	
	self 
		longitude: nil;
		latitude: nil;
		zoom: nil.! !

!GMStaticMapExampleBase methodsFor: 'private' stamp: 'RichardStern 12/6/2010 14:54'!
centerOnMapPoint: aGMMapPoint
	
	self 
		longitude: aGMMapPoint longitude;
		latitude: aGMMapPoint latitude.! !

!GMStaticMapExampleBase methodsFor: 'private' stamp: 'RichardStern 2/17/2015 09:56'!
createProjection
	"Utitlity method to create and return a projection object for current map"
	
	| proj |

	proj := GMStaticMapMercatorProjection new.
	proj
		centerCoordinate: (GMStaticMapPoint new 
								latitude: self latitude asNumber;
								longitude: self longitude asNumber;
								yourself);							
		zoom: self zoom asNumber;
		imageWidth: self width asNumber;
		imageHeight: self height asNumber.
		
	^proj.! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:56'!
height
	^ height! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:57'!
height: anObject
	height := anObject! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 3/29/2010 16:07'!
imageFormat
	^ imageFormat! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 3/29/2010 16:07'!
imageFormat: anObject
	imageFormat := anObject! !

!GMStaticMapExampleBase methodsFor: 'initializing' stamp: 'rs 12/11/2013 12:52'!
initialize
	super initialize.				

	self resetMap.
! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'RichardStern 4/17/2014 14:36'!
language
	^ language! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'RichardStern 4/17/2014 14:36'!
language: anObject
	language := anObject! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:57'!
latitude
	^ latitude! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:57'!
latitude: anObject
	latitude := anObject! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:57'!
longitude
	^ longitude! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:57'!
longitude: anObject
	longitude := anObject! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 7/25/2013 08:40'!
mapType
	^ mapType! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 7/25/2013 08:40'!
mapType: anObject
	mapType := anObject! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 13:23'!
markers
	^markers! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 13:24'!
markers: anObject
	markers := anObject! !

!GMStaticMapExampleBase methodsFor: 'actions' stamp: 'rs 12/11/2013 12:51'!
removeAllMarkers

	self markers: OrderedCollection new.! !

!GMStaticMapExampleBase methodsFor: 'actions' stamp: 'RichardStern 1/3/2011 15:47'!
removeAllMarkersSatisfying: aBlock

	self markers: (self markers reject: aBlock).! !

!GMStaticMapExampleBase methodsFor: 'actions' stamp: 'rs 12/11/2013 12:51'!
removeAllStyles

	self styles: OrderedCollection new.
! !

!GMStaticMapExampleBase methodsFor: 'rendering-utiltiy' stamp: 'RichardStern 2/12/2016 08:06'!
renderCheckboxFor: aSymbol label: aLabelString hint: aHintString model: aModel on: html
	"Utility method used to render a checkbox for a parameter in unified way"

	html tableRow:
	[
		html tableData:
		[
			html label 
				for: aSymbol;
				with: aLabelString.
		].
	
		html tableData:
		[
			html text: aHintString. 
		].


		html tableData:
		[		
			html checkbox
		            on: aSymbol 
						of: aModel.				
		].
	].! !

!GMStaticMapExampleBase methodsFor: 'rendering' stamp: 'rs 12/4/2013 08:47'!
renderContentOn: html
	"Render three  areas that are the basic building blocks of the examples:
		- a text areas with example descritpion on top
		- an area for parameter input forms on the left and 
		- a pane displaying the resulting map one the right"

	html div
		class: 'description';
		with: [ self renderDescriptionOn: html.].	
	html div style: 'clear: both'.
					
	html div
		class: 'inputForm';
		with: [ self renderFormsOn: html].	
			
	html div
		class: 'map';
		with: [ self renderMapOn: html].
		! !

!GMStaticMapExampleBase methodsFor: 'rendering' stamp: 'rs 12/3/2013 10:04'!
renderDescriptionOn: html
	"Render a description of the example. Do nothing by default - subclasses may override properly"
	
	
! !

!GMStaticMapExampleBase methodsFor: 'rendering-utiltiy' stamp: 'RichardStern 2/11/2016 11:18'!
renderDropdownFor: aSymbol label: aLabelString hint: aHintString model: aModel withValues: aCollection on: html
	"Utility method used to render a dropdown field for a parameter in unified way"
	
	html tableRow:
	[
		html tableData:
		[
			html label 
				for: aSymbol;
				with: aLabelString.
		].
	
		html tableData:
		[
			html text: aHintString. 
		].


		html tableData:
		[		
			html select
		         list: aCollection;
					selected: (aModel perform: aSymbol);
		         on: aSymbol of: aModel.
		].
	].! !

!GMStaticMapExampleBase methodsFor: 'rendering' stamp: 'rs 12/3/2013 09:21'!
renderFormsOn: html
	"Render the form(s) used to parametrize the map. By default, a form for the basic map parameters (position, 
	zoom)... but subclasses my override / extend this according to their needs"

	self renderMapFormOn: html.
	html break.
! !

!GMStaticMapExampleBase methodsFor: 'rendering-map form' stamp: 'RichardStern 4/18/2014 08:05'!
renderImageFormatSelectionOn: html
	"Render dropdown box allowing to choose map image format"
	
	html tableRow:
	[
		html tableData:
		[
			html label 
				for: #imageFormat;
				with: 'Image format'.
		].
	
		html tableData:
		[
			html text: '-'. 
		].

		html tableData:
		[		
			html select
				list: (GMStaticMap supportedImageFormats);
				selected: self imageFormat;
				on: #imageFormat of: self.

		].
	].! !

!GMStaticMapExampleBase methodsFor: 'rendering-utiltiy' stamp: 'rs 12/3/2013 09:25'!
renderInputCaption: aString on: html
	"Render a caption for parameter input table used in the examples"
	
	html tableData
		class: 'caption';
		with:
		[
			html text: aString.
		].
! !

!GMStaticMapExampleBase methodsFor: 'rendering-utiltiy' stamp: 'rs 12/3/2013 09:24'!
renderInputCaptionsOn: html
		"Render heading for parameter input table used in the examples"
		
		html tableHead:
		[
			self 
				renderInputCaption: 'Parameter' on: html;
				renderInputCaption: 'Hint' on: html;
				renderInputCaption: 'Value' on: html.																					
		].
			
! !

!GMStaticMapExampleBase methodsFor: 'rendering-utiltiy' stamp: 'RichardStern 2/11/2016 11:14'!
renderInputFor: aSymbol label: aLabelString hint: aHintString model: aModel on: html
	"Utility method used to render a text input field for a parameter in unified way"

	html tableRow:
	[
		html tableData:
		[
			html label 
				for: aSymbol;
				with: aLabelString.
		].
	
		html tableData:
		[
			html text: aHintString. 
		].

		html tableData:
		[		
			html textInput 
				size: 18;
				on: aSymbol of: aModel.
		].
	].! !

!GMStaticMapExampleBase methodsFor: 'rendering-map form' stamp: 'RichardStern 2/12/2016 07:49'!
renderLanguageSelectionOn: html
	"Render dropdown box allowing to choose language"
	
	self
		renderDropdownFor: #language 
		label: 'Language' 
		hint: '-' 
		model: self
		withValues: (GMStaticMap supportedLanguages) 
		on: html.	
			! !

!GMStaticMapExampleBase methodsFor: 'rendering-map form' stamp: 'RichardStern 4/17/2014 14:29'!
renderMapFormOn: html
	"Render a form allowing ot change basic map parameters"
	
	html heading
			level: 5;
			with: 'Map'.

	html form
		with:
		[			
			html table:
			[
				self renderInputCaptionsOn: html.
			
				self 
					renderMapInputFor: #longitude label: 'Longitude' hint: '-' on: html;
					renderMapInputFor: #latitude label: 'Latitude' hint: '-' on: html;				
					renderMapInputFor: #zoom label: 'Zoom' hint: '0 - 19' on: html;
					renderMapInputFor: #width label: 'Map width' hint: 'max 640' on: html;
					renderMapInputFor: #height label: 'Map height' hint: 'max 640' on: html;
					renderMapTypeSelectionOn: html;
					renderMapScaleSelectionOn: html;
					renderImageFormatSelectionOn: html;
					renderLanguageSelectionOn: html.
			].		
					
			html cancelButton 
					value: 'Reset';
					callback: [self resetMap].
			html submitButton 
					value: 'Update'.						
			html submitButton 
					value: 'Zoom in';
					callback: [self zoomIn].
			html submitButton 
					value: 'Zoom out';
					callback: [self zoomOut].											
		].
		
! !

!GMStaticMapExampleBase methodsFor: 'rendering-map form' stamp: 'rs 3/29/2010 14:39'!
renderMapInputFor: aSymbol label: aLabelString hint: aHintString on: html
	"Utility method used to render all the input fields in example in unified way"
	
	self 
		renderInputFor: aSymbol 
		label: aLabelString 
		hint: aHintString 
		model: self
		on: html! !

!GMStaticMapExampleBase methodsFor: 'rendering' stamp: 'RichardStern 4/18/2014 08:05'!
renderMapOn: html
	"Render map itself"
	"Note that longitude, latitude and zoom cannot be assumed to be numbers as they might be nil in case that
	map is implicitly positioned by the markers - so no conversion for them!!"
	
	^html googleStaticMap
		longitude: self longitude;
		latitude: self latitude;
		zoom: self zoom;
		width: self width asNumber;
		height: self height asNumber;
		mapType: self mapType asString;
		scale: self scale asNumber;
		language: self language;
		imageFormat: self imageFormat;
		addMarkerGroups: self markers;
		addStyles: self styles;
		yourself.

! !

!GMStaticMapExampleBase methodsFor: 'rendering-map form' stamp: 'RichardStern 2/11/2016 11:27'!
renderMapScaleSelectionOn: html
	"Render checkbox allowing to choose scale value for map (usually needed 
	 for mobile devices)"
	
	self
		renderDropdownFor: #scale 
		label: 'Scale' 
		hint: '4 only for enterprise use' 
		model: self
		withValues: (GMStaticMap supportedScaleValues) 
		on: html.	! !

!GMStaticMapExampleBase methodsFor: 'rendering-map form' stamp: 'RichardStern 2/11/2016 11:26'!
renderMapTypeSelectionOn: html
	"Render dropdown box allowing to choose map type"
	
	self
		renderDropdownFor: #mapType 
		label: 'Map type' 
		hint: '-' 
		model: self
		withValues: (GMStaticMap supportedMapTypes) 
		on: html.! !

!GMStaticMapExampleBase methodsFor: 'actions' stamp: 'rs 12/11/2013 13:11'!
resetMap
	"Reset any configurations stored in receiver that affect map display"

	self 
		"Beware to reset markers and styles before invoking #setInitialParameters to allow
		initial markers and styles to be set in that method"
		removeAllMarkers;
		removeAllStyles;
		setInitialParameters.

! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 7/29/2013 08:13'!
scale
	^ scale! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 7/29/2013 08:13'!
scale: anObject
	scale := anObject! !

!GMStaticMapExampleBase methodsFor: 'private' stamp: 'RichardStern 2/11/2016 11:12'!
setInitialParameters
	"Set initial paramters to use for the example. This is called both on initialization and by
	the map reset buttons. Subclasses should override according to their needs"
	
	longitude := '16.3725'.
	latitude := '48.2085'.
	zoom := '5'.
	width := '500'.
	height := '500'.
	mapType := 'roadmap'.
	scale := 1.
	language := 'en'.! !

!GMStaticMapExampleBase methodsFor: 'rendering' stamp: 'rs 12/11/2013 09:01'!
style

^'
div.description {float: left; clear: both; width: 90%; max-width: 50em; margin-bottom: 1em; padding: 4px; background-color: lightYellow; border: 1px solid gray;}
div.inputForm {float: left; margin-right: 20px;}
div.map {flow: right;}
div.map img {vertical-align: middle;}
div.inputForm table { border: 1px solid gray; background-color: #e0e0e0; margin-bottom: 0.5em;}
div.inputForm td {padding-left: 5px; padding-right: 15px; font-size:0.8em;}
div.inputForm h5{margin-top: 4px; margin-bottom: 8px;}
td.caption {font-weight: bold; background-color: #c0c0c0}
'.

		! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 12/11/2013 12:50'!
styles
	^ styles! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 12/11/2013 12:50'!
styles: anObject
	styles := anObject! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:57'!
width
	^ width! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:58'!
width: anObject
	width := anObject! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:58'!
zoom
	^ zoom! !

!GMStaticMapExampleBase methodsFor: 'accessing' stamp: 'rs 1/21/2010 12:58'!
zoom: anObject
	zoom := anObject! !

!GMStaticMapExampleBase methodsFor: 'actions' stamp: 'rs 1/21/2010 13:07'!
zoomIn
	| zoomLevel |
	
	zoomLevel := self zoom asInteger.
	zoomLevel := ((zoomLevel + 1) min: 19) max: 0.
	
	self zoom: zoomLevel asString.! !

!GMStaticMapExampleBase methodsFor: 'actions' stamp: 'rs 1/21/2010 13:07'!
zoomOut
	| zoomLevel |
	
	zoomLevel := self zoom asInteger.
	zoomLevel := ((zoomLevel - 1) max: 0) min: 19.
	
	self zoom: zoomLevel asString.! !

GMStaticMapExampleBase subclass: #GMStaticMapExampleMarkers
	instanceVariableNames: 'standardMarker customMarker'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

!GMStaticMapExampleMarkers methodsFor: 'actions' stamp: 'RichardStern 1/3/2011 15:22'!
addCustomMarker

	self 
		addMarker: self customMarker;
		resetCustomMarker.
	! !

!GMStaticMapExampleMarkers methodsFor: 'actions' stamp: 'RichardStern 1/3/2011 15:55'!
addMarker: aGMMarkerGroup

	self markers add: aGMMarkerGroup.
	self centerOnMapPoint: aGMMarkerGroup points first.

	! !

!GMStaticMapExampleMarkers methodsFor: 'actions' stamp: 'RichardStern 1/3/2011 15:22'!
addStandardMarker
	self 
		addMarker: self standardMarker;
		resetStandardMarker.! !

!GMStaticMapExampleMarkers methodsFor: 'accessing' stamp: 'RichardStern 1/3/2011 15:18'!
customMarker
	^ customMarker! !

!GMStaticMapExampleMarkers methodsFor: 'accessing' stamp: 'RichardStern 1/3/2011 15:18'!
customMarker: anObject
	customMarker := anObject! !

!GMStaticMapExampleMarkers methodsFor: 'actions' stamp: 'RichardStern 1/3/2011 15:45'!
removeAllCustomMarkers
	self removeAllMarkersSatisfying: [:m | m isKindOf: GMStaticMapIconMarkerGroup]! !

!GMStaticMapExampleMarkers methodsFor: 'actions' stamp: 'RichardStern 1/3/2011 15:32'!
removeAllStandardMarkers
	self removeAllMarkersSatisfying: [:m | m isKindOf: GMStaticMapStandardMarkerGroup]! !

!GMStaticMapExampleMarkers methodsFor: 'rendering' stamp: 'rs 12/4/2013 08:50'!
renderCustomMarkerFormOn: html

	html heading
			level: 5;
			with: 'Custom Markers'.

	html form:
	[
		html table:
		[
			self renderInputCaptionsOn: html.
			
			self 
				renderInputFor: #longitude label: 'Longitude' hint: '-' model: self customMarker firstPoint on: html;
				renderInputFor: #latitude label: 'Latitude' hint: '-' model: self customMarker firstPoint on: html;					
				renderInputFor: #icon label: 'Icon' hint: 'Url of marker icon' model: self customMarker on: html;
				renderCustomMarkerShadowSelectionOn: html.
		].
	
		html cancelButton 
				value: 'Remove all custom markers';
				callback: [self removeAllCustomMarkers].
		html submitButton 
				value: 'Add custom marker';
				callback: [self addCustomMarker].				

	].
		! !

!GMStaticMapExampleMarkers methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 08:12'!
renderCustomMarkerShadowSelectionOn: html
	"Render checkbox allowing to choose if custom marker should have shadow"
	
	self
		renderCheckboxFor: #shadow
		label: 'Shadow' 
		hint: 'Affects used center!!' 
		model: self customMarker 
		on: html.		! !

!GMStaticMapExampleMarkers methodsFor: 'rendering' stamp: 'rs 12/11/2013 08:28'!
renderDescriptionOn: html

	html 
		text: 'Markers exist in two basic fashions - standard ones displaying a single character and using different colors and custom ones which allow using a dedicated icon. Just try the "Add marker" and "Add custom marker" buttons using the pre-filled values and get a standard marker for Vienna and a custom marker at Munich.'.

! !

!GMStaticMapExampleMarkers methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 07:55'!
renderFormsOn: html

	self renderMapFormOn: html.
	html break.
	self renderStandardMarkerFormOn: html.
	html break.
	self renderCustomMarkerFormOn: html.
	html break; break.! !

!GMStaticMapExampleMarkers methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 07:54'!
renderStandardMarkerColorSelectionOn: html
	"Render out a drop down allowing to select marker color"

	self
		renderDropdownFor: #color 
		label: 'Color' 
		hint: '-' 
		model: self standardMarker
		withValues: (GMStaticMapStandardMarkerGroup supportedColors) 
		on: html.	
				! !

!GMStaticMapExampleMarkers methodsFor: 'rendering' stamp: 'RichardStern 1/3/2011 15:30'!
renderStandardMarkerFormOn: html

	html heading
			level: 5;
			with: 'Standard markers'.

	html form:
	[
		html table:
		[
			self renderInputCaptionsOn: html.
			
			self 
				renderInputFor: #longitude label: 'Longitude' hint: '-' model: self standardMarker firstPoint on: html;
				renderInputFor: #latitude label: 'Latitude' hint: '-' model: self standardMarker firstPoint on: html;				
				renderInputFor: #label label: 'Label' hint: 'single char' model: self standardMarker on: html;
				renderStandardMarkerColorSelectionOn: html;
				renderStandardMarkerSizeSelectionOn: html.
		].
	
		html cancelButton 
				value: 'Remove all standard markers';
				callback: [self removeAllStandardMarkers].
		html submitButton 
				value: 'Add marker';
				callback: [self addStandardMarker].				

	].
		! !

!GMStaticMapExampleMarkers methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 07:53'!
renderStandardMarkerSizeSelectionOn: html
	"Render out a drop down allowing to select marker size"

	self
		renderDropdownFor: #size 
		label: 'Size' 
		hint: '-' 
		model: self standardMarker
		withValues: (GMStaticMapStandardMarkerGroup supportedSizes) 
		on: html.	
				! !

!GMStaticMapExampleMarkers methodsFor: 'private' stamp: 'rs 12/4/2013 08:45'!
resetCustomMarker
	self customMarker: GMStaticMapIconMarkerGroup new.
	self customMarker
			icon: 'http://chart.apis.google.com/chart?chst=d_map_pin_icon&chld=cafe';
			shadow: true;
			addPointWithLongitude: 11.3501 andLatitude: 48.0818.! !

!GMStaticMapExampleMarkers methodsFor: 'private' stamp: 'rs 12/4/2013 08:45'!
resetStandardMarker
	self standardMarker: GMStaticMapStandardMarkerGroup new.	
	self standardMarker 
			color: 'red';
			label: (self markers size+1) asString;
			addPointWithLongitude: 16.3725 andLatitude: 48.2085.! !

!GMStaticMapExampleMarkers methodsFor: 'private' stamp: 'RichardStern 1/3/2011 15:40'!
setInitialParameters
	
	super setInitialParameters.
	self 
		resetStandardMarker;
		resetCustomMarker.
! !

!GMStaticMapExampleMarkers methodsFor: 'accessing' stamp: 'RichardStern 1/3/2011 15:20'!
standardMarker
	^standardMarker! !

!GMStaticMapExampleMarkers methodsFor: 'accessing' stamp: 'RichardStern 1/3/2011 15:20'!
standardMarker: anObject
	standardMarker := anObject! !

GMStaticMapExampleBase subclass: #GMStaticMapExampleMove
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

!GMStaticMapExampleMove methodsFor: 'private' stamp: 'RichardStern 07/19/2015 04:27'!
addOrientationMarkers
	"Add markers at the borders of initial map (actually a 95 % of it for sake of visiblity) to make moves more apparent"
	
	| proj deltaX deltaY markerGroup |

	"Calculate deltas of the markers from center"
	deltaX := (self width asInteger / 2 * 0.95) rounded.
	deltaY := (self height asInteger / 2 * 0.95) rounded.

	"Prepare a marker group"
	markerGroup := GMStaticMapStandardMarkerGroup  new
						label: 'X';
						color: 'red';
						size: 'normal';
						yourself.

	"Use projection to calculate marker coordinates"
	proj := self createProjection.
	#(
		#(-1 -1) #(0 -1) #(1 -1) 
		#(-1 0) #(1 0) 
		#(-1 1) #(0 1) #(1 1)
	) do:
	[ :pair | | delta mapPoint |
		delta := Point x: (pair first * deltaX) y: (pair last * deltaY).
		mapPoint := proj coordinateOfPointWithCenterDelta: delta.
		markerGroup addPoint: mapPoint.
	].

	"Finally, add the marker group to map"
	self markers: (OrderedCollection with: markerGroup).


	
! !

!GMStaticMapExampleMove methodsFor: 'actions' stamp: 'RichardStern 2/17/2015 10:00'!
moveToAdjacentMapInDirection: aSymbol

	| coord |

	"Make a projection to get the new center coordinate when moving in given direction"
	coord := self createProjection centerCoordinateOfAdjacentMapInDirection: aSymbol.

	"Set new center for map"
	self 
		longitude: coord longitude;
		latitude: coord latitude.

! !

!GMStaticMapExampleMove methodsFor: 'private' stamp: 'RichardStern 2/17/2015 10:26'!
renderDescriptionOn: html

	html 
		text: 'Mercator projection also allows navigation to adjacent maps - use the cardinal direction links to move around. For better orientation, markers indicate the bounds of the original map (actually 95 % of original extent to keep markers visible)'.

! !

!GMStaticMapExampleMove methodsFor: 'rendering' stamp: 'RichardStern 2/17/2015 08:55'!
renderFormsOn: html

	self renderMapFormOn: html.
	html break.
	self renderMoveFormOn: html.
	html break.
! !

!GMStaticMapExampleMove methodsFor: 'rendering' stamp: 'RichardStern 07/19/2015 04:19'!
renderLinkCellForDirection: aSymbol withLabel: aString on: html

	html tableData 
		with:
		[ 
			html anchor 	
				callback: [ self moveToAdjacentMapInDirection: aSymbol. ];
				with: aString.
		].
! !

!GMStaticMapExampleMove methodsFor: 'rendering' stamp: 'RichardStern 2/18/2015 08:28'!
renderMoveFormOn: html

	html heading
			level: 5;
			with: 'Move'.
			
	html table 
		class: 'dirLinks';
		with:
		[ 
			html tableRow with:
			[ 
				self 
					renderLinkCellForDirection: #NorthWest withLabel: 'NW' on: html;
					renderLinkCellForDirection: #North withLabel: 'N' on: html;
					renderLinkCellForDirection: #NorthEast withLabel: 'NE' on: html.
			].
			html tableRow with:
			[ 
				self 
					renderLinkCellForDirection: #West withLabel: 'W' on: html;
					renderLinkCellForDirection: #None withLabel: '*' on: html;
					renderLinkCellForDirection: #East withLabel: 'E' on: html.
			].
				html tableRow with:
			[ 
				self 
					renderLinkCellForDirection: #SouthWest withLabel: 'SW' on: html;
					renderLinkCellForDirection: #South withLabel: 'S' on: html;
					renderLinkCellForDirection: #SouthEast withLabel: 'NE' on: html.
			].
		].
! !

!GMStaticMapExampleMove methodsFor: 'private' stamp: 'RichardStern 2/17/2015 09:29'!
setInitialParameters

	super setInitialParameters.
	self addOrientationMarkers.! !

!GMStaticMapExampleMove methodsFor: 'rendering' stamp: 'RichardStern 2/18/2015 08:32'!
style

^ super style, 
'
	table.dirLinks { border-collapse: collapse; }
	table.dirLinks td { text-align:center; padding: 2em; border: 1px dotted #CCC}
	table.dirLinks td a { text-decoration: none; font-weight: bold; color: #000; }
'
	

! !

GMStaticMapExampleBase subclass: #GMStaticMapExamplePath
	instanceVariableNames: 'path pathPoints selectedPathPoints pathPointLongitude pathPointLatitude random'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

!GMStaticMapExamplePath methodsFor: 'actions' stamp: 'RichardStern 2/11/2016 10:38'!
addPathPoint
	| p  | 

	p := self pathPoint.
	self pathPoints add: p.

	"Automatically add new point to selected ones and update map"
	self selectedPathPoints add: p.
	self showPath. 
	
	"For convenience, calculate coordinates for a new random point and suggest them for next add"
	self suggestRandomPathPointStartingFrom: p.
	! !

!GMStaticMapExamplePath methodsFor: 'initializing' stamp: 'rs 12/10/2013 16:29'!
initialize
	super initialize.

	pathPoints  := OrderedCollection new.
	selectedPathPoints  := OrderedCollection new.
	random := Random seed: 314159.! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 13:01'!
path
	^ path! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 13:01'!
path: anObject
	path := anObject! !

!GMStaticMapExamplePath methodsFor: 'private' stamp: 'RichardStern 2/11/2016 10:07'!
pathPoint
	"Create and return a map point based on the coordinates in path point form"

	^GMStaticMapPoint new
					longitude: (self pathPointLongitude asNumber);
					latitude: (self pathPointLatitude asNumber);
					yourself.! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 15:01'!
pathPointLatitude
	^ pathPointLatitude! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 15:01'!
pathPointLatitude: anObject
	pathPointLatitude := anObject! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 15:01'!
pathPointLongitude
	^ pathPointLongitude! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 15:01'!
pathPointLongitude: anObject
	pathPointLongitude := anObject! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 15:25'!
pathPoints
	^ pathPoints! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 15:25'!
pathPoints: anObject
	pathPoints := anObject! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 12/10/2013 16:28'!
random
	^ random! !

!GMStaticMapExamplePath methodsFor: 'rendering' stamp: 'rs 12/10/2013 16:25'!
renderDescriptionOn: html

	html 
		text: 'Paths allow displaying of polines (e.g. a route) or even filled polygons and are therefore a more complex feature of the Google Static maps API. Try to add some coordinates using the form at the bottom of the page and the use the form in the middle to modify the appearance of the line. For a quick try, you may also use the "'.
	html anchor
		callback: [self addPathPoint.];
		with: 'Add Point'.
	html text: '"-Button several times in succession as it modifies the coordinates a little bit on each activation to allow for a quick impression.'.! !

!GMStaticMapExamplePath methodsFor: 'rendering' stamp: 'RichardStern 2/11/2016 10:01'!
renderFormsOn: html

	self renderMapFormOn: html.
	html break.	
	self renderPathFormOn: html.
	html break.
	self renderPathPointsFormOn: html.
	html break; break.! !

!GMStaticMapExamplePath methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 08:10'!
renderGeodesicSelectionOn: html
	"Render checkboxallowing to choose if path is geodesic"
	
	self
		renderCheckboxFor: #geodesic
		label: 'Geodesic' 
		hint: '-' 
		model: self path 
		on: html.	
			
! !

!GMStaticMapExamplePath methodsFor: 'rendering' stamp: 'rs 8/9/2013 09:59'!
renderMapOn: html

	^(super renderMapOn: html)
			path: self path;
			yourself.
! !

!GMStaticMapExamplePath methodsFor: 'rendering' stamp: 'RichardStern 2/11/2016 11:04'!
renderPathFormOn: html

	html heading
			level: 5;
			with: 'Path'.

	html form:
	[
		html table:
		[
			self renderInputCaptionsOn: html.

			self 
				renderPathInputFor: #color 
				label: 'Color' 
				hint: 'RBG color value' 
				on: html.
			self 
				renderPathInputFor: #fillcolor 
				label: 'Fill color' 
				hint: 'RBG color value' 
				on: html.
			self 
				renderPathInputFor: #weight 
				label: 'Line width' 
				hint: 'in Pixels' 
				on: html.
			self renderGeodesicSelectionOn: html.
										
			html tableRow:
			[
				html tableData:
				[
					html label with: 'Paths points'.
				].
	
				html tableData: [html text: 'GPS Points'; break; text: '(select here, enter below)'].

				html tableData:
				[		
					html multiSelect
						size: 3;
						list: self pathPoints;
						labels: [:p | p displayString];
						"For convenience, select all automatically if user did not make an selection himself yet"
						selected: (self selectedPathPoints isEmpty 
											ifTrue: [self pathPoints] 
											ifFalse: [self selectedPathPoints]);
						callback: [ :value | self selectedPathPoints: value].
				].
			].		

		].
	
		html cancelButton 
				value: 'Reset';
				callback: [self resetMap].
					
		html submitButton 
				value: 'Show path';
				callback: [self showPath].				

	].
		
! !

!GMStaticMapExamplePath methodsFor: 'rendering' stamp: 'rs 1/21/2010 13:20'!
renderPathInputFor: aSymbol label: aLabelString hint: aHintString on: html

	self 
		renderInputFor: aSymbol 
		label: aLabelString 
		hint: aHintString 
		model: self path
		on: html
! !

!GMStaticMapExamplePath methodsFor: 'rendering' stamp: 'rs 1/21/2010 15:29'!
renderPathPointsFormOn: html

	html heading
			level: 5;
			with: 'Path points'.

	html form:
	[
		html table:
		[
			self 
				renderMapInputFor: #pathPointLongitude label: 'Longitude' hint: '-' on: html;
				renderMapInputFor: #pathPointLatitude label: 'Latitude' hint: '-' on: html.	
		].

		html submitButton 
			value: 'Add point ';
			callback: [self addPathPoint].	

	].
		
! !

!GMStaticMapExamplePath methodsFor: 'actions' stamp: 'RichardStern 12/6/2010 12:55'!
resetPath
	path := GMStaticMapPath new.	
	self path
			color: '0xff0000ff';
			fillcolor: '0x0000ff';
			weight: '3'.
				
	self 
		pathPoints: OrderedCollection new;
		selectedPathPoints: OrderedCollection new.
		! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 15:04'!
selectedPathPoints
	^ selectedPathPoints! !

!GMStaticMapExamplePath methodsFor: 'accessing' stamp: 'rs 1/21/2010 15:04'!
selectedPathPoints: anObject
	selectedPathPoints := anObject! !

!GMStaticMapExamplePath methodsFor: 'private' stamp: 'rs 3/29/2010 14:57'!
setInitialParameters
	
	super setInitialParameters.
	self resetPath.
						
	self pathPointLongitude: self longitude.
	self pathPointLatitude: self latitude.

! !

!GMStaticMapExamplePath methodsFor: 'actions' stamp: 'rs 3/29/2010 15:38'!
showPath

	self selectedPathPoints isEmpty ifTrue:
	[
		^self inform: 'No path points selected!!'.
	].
	
	self path resetPoints.
	self selectedPathPoints do: [:m | self path addPoint: m].
			! !

!GMStaticMapExamplePath methodsFor: 'private' stamp: 'RichardStern 2/11/2016 10:41'!
suggestRandomPathPointStartingFrom: aGMStaticMapPoint
	"Assume passed point is last one on a path. Now, create a random point in its vicinity 
	and put its longitude/latitude into the path point for input values."
	"Note: For a good user experience, next point should be on shown map - to achieve this,
	make use of now available mercator projection."
		
	| xDelta yDelta signX signY mapPoint |
	signX := (self random next > 0.5) 
					ifTrue: [1] 
					ifFalse: [-1].
	signY := (self random next > 0.5) 
					ifTrue: [1] 
					ifFalse: [-1].

	xDelta := signX * (self random nextInt: (self width / 4)).
	yDelta := signY * (self random nextInt: (self height / 4)).
	
	mapPoint := self createProjection coordinateOfPointWithCenterDelta: (Point x: xDelta y: yDelta).
								
	self 
		pathPointLongitude: (mapPoint longitude);
		pathPointLatitude: (mapPoint latitude).! !

GMStaticMapExampleBase subclass: #GMStaticMapExamplePlainMap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

!GMStaticMapExamplePlainMap methodsFor: 'rendering' stamp: 'rs 12/11/2013 08:27'!
renderDescriptionOn: html

	html text: 'This example focuses on the basic display of a static map - just try all the different parameters on the right side area to influence position, zoom level, size map type etc. - or simple use one of the following links to get a '.
	
	html anchor
		callback: 
		[	
			self
				longitude: '16.3725';
				latitude: '48.2085';
				zoom: '17';
				width: '500';
				height: '500';
				mapType: 'hybrid';
				scale: 1.
		];
		with: 'hybrid view of Vienna city'.
	html text: ' or a '.
		html anchor
		callback: 
		[
			self
				longitude: '12.50';
				latitude: '41.88';
				zoom: '11';
				width: '350';
				height: '350';
				mapType: 'terrain';
				scale: 1.
		];
		with: 'terrain map of Rome'.
	html text: '.'
	
	
	
	
! !

GMStaticMapExampleBase subclass: #GMStaticMapExampleStyledMap
	instanceVariableNames: 'currentStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

!GMStaticMapExampleStyledMap methodsFor: 'actions' stamp: 'rs 12/11/2013 13:43'!
addStyle
	"Take over current style to applied ones and establish a plain new style object as current one"

	self styles add: self currentStyle.
	self resetCurrentStyle.! !

!GMStaticMapExampleStyledMap methodsFor: 'actions' stamp: 'rs 12/11/2013 13:32'!
applyStyle: aGMStaticMapStyle
	"Reset current styling of map and apply passed style. This is a utility method used to show styles not entered via form, but
	given programmatically"

	self removeAllStyles.
	self styles add: aGMStaticMapStyle.
! !

!GMStaticMapExampleStyledMap methodsFor: 'accessing' stamp: 'rs 12/11/2013 08:43'!
currentStyle
	^ currentStyle! !

!GMStaticMapExampleStyledMap methodsFor: 'accessing' stamp: 'rs 12/11/2013 08:43'!
currentStyle: anObject
	currentStyle := anObject! !

!GMStaticMapExampleStyledMap methodsFor: 'initialize-release' stamp: 'rs 12/11/2013 12:50'!
initialize 	

	super initialize.
	
	self resetCurrentStyle.
! !

!GMStaticMapExampleStyledMap methodsFor: 'rendering' stamp: 'rs 12/11/2013 13:59'!
renderDescriptionOn: html

	html text: 'Styles allow customization of different map contents (labels, lines etc.). This is done by declaring one or more style rules each of which apply to a certain type of map elements. Style thus can e.g. be used to achieve '.
	
	html unorderedList:
	[
		html listItem:
		[
			html anchor
				callback: 
				[	
					self applyStyle:
							(GMStaticMapStyle new
									feature: 'all';
									element: 'labels';
									visibility: 'off';
									yourself).
				];
				with: 'a map without any labels,'.
		].
		html listItem:
		[
			html anchor
				callback: 
				[	
					self applyStyle:
							(GMStaticMapStyle new
									feature: 'road.highway';
									element: 'geometry';
									hue: '0x2000ff';
									yourself).
				];
				with: 'highways displayed in a custom color,'.
		].
		html listItem:
		[
			html anchor
				callback: 
				[	
					self applyStyle:
							(GMStaticMapStyle new
									feature: 'poi.park';
									element: 'all';
									hue: '0xff0000';
									saturation: '100';								
									yourself).
				];
				with: 'all parks in red'.
		].	
	].

	html text: 'and even any combination of such effects by using multiple styles.'.
! !

!GMStaticMapExampleStyledMap methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 07:58'!
renderElementSelectionOn: html
	"Render out a drop down allowing the select the element the entered style should apply to"
	
	self
		renderDropdownFor: #element 
		label: 'Element' 
		hint: 'Styling target' 
		model: self currentStyle
		withValues: (GMStaticMapStyle supportedElements) 
		on: html.	
				
				! !

!GMStaticMapExampleStyledMap methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 07:59'!
renderFeatureSelectionOn: html
	"Render out a drop down allowing the select the feature the entered style should apply to"

	self
		renderDropdownFor: #feature 
		label: 'Feature' 
		hint: 'Styling target' 
		model: self currentStyle
		withValues: (GMStaticMapStyle supportedFeatures) 
		on: html.					! !

!GMStaticMapExampleStyledMap methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 07:56'!
renderFormsOn: html

	self renderMapFormOn: html.
	html break.
	self renderStyleDefinitionFormOn: html.
	html break.
	self renderStyleUsageFormOn: html.
	html break; break.
! !

!GMStaticMapExampleStyledMap methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 08:08'!
renderInverseLightnessSelectionOn: html
	"Render checkbox allowing to choose if lightness should be inversed"
	
	self
		renderCheckboxFor: #inverseLightness
		label: 'Inverse Lightness' 
		hint: '-' 
		model: self currentStyle 
		on: html.	! !

!GMStaticMapExampleStyledMap methodsFor: 'rendering' stamp: 'rs 12/11/2013 11:22'!
renderStyleDefinitionFormOn: html

	html heading
			level: 5;
			with: 'Style'.

	html form:
	[
		html table:
		[
			self renderInputCaptionsOn: html.
			
			self 
				renderFeatureSelectionOn: html;
				renderElementSelectionOn: html;
				renderInputFor: #hue label: 'Hue' hint: 'Format: 0xRRGGBB' model: self currentStyle on: html;
				renderInputFor: #lightness label: 'Lightness' hint: '-100 to 100' model: self currentStyle on: html;				
				renderInputFor: #saturation label: 'Saturation' hint: '-100 to 100' model: self currentStyle on: html;										
				renderInputFor: #gamma label: 'Gamma' hint: '0.1 to 10' model: self currentStyle on: html;
				renderInverseLightnessSelectionOn: html;
				renderVisibilitySelectionOn: html.
		].
	
		html submitButton 
				value: 'Add style';
				callback: [self addStyle].				

	].
		! !

!GMStaticMapExampleStyledMap methodsFor: 'rendering' stamp: 'rs 12/11/2013 13:16'!
renderStyleUsageFormOn: html
	| output |
	
	html heading
			level: 5;
			with: 'Applied Styles'.

	output := WriteStream on: String new.
	self styles do: [:s | s asUrlParameterOn: output. output crlf.].
	
	html form:
	[
		html div 
			class: 'usedStyleRules';
			with:
			[
				html teletype: output contents.
			].

		html submitButton 
				value: 'Reset styles';
				callback: [self removeAllStyles.].				
	].

! !

!GMStaticMapExampleStyledMap methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 08:01'!
renderVisibilitySelectionOn: html
	"Render out a drop down allowing to visibility"

	self
		renderDropdownFor: #visibility 
		label: 'Visibility' 
		hint: '-' 
		model: self currentStyle
		withValues: (GMStaticMapStyle supportedVisibilityValues) 
		on: html.						! !

!GMStaticMapExampleStyledMap methodsFor: 'actions' stamp: 'rs 12/11/2013 11:32'!
resetCurrentStyle

	self currentStyle: GMStaticMapStyle new.! !

!GMStaticMapExampleStyledMap methodsFor: 'private' stamp: 'rs 12/11/2013 13:51'!
setInitialParameters
	"Override superclass to set a different zoom level"
	
	super setInitialParameters.
	self zoom: '13'.
! !

!GMStaticMapExampleStyledMap methodsFor: 'rendering' stamp: 'rs 12/11/2013 12:26'!
style
	^super style,
	'
	div.usedStyleRules
	{	border: 1px solid #c0c0c0;
		margin-top: 1em;
		margin-bottom: 0.5em;
		overflow: auto;
		width: 50em;
		height: 8em;
		font-size: 7pt;
	}

	'! !

GMStaticMapExampleBase subclass: #GMStaticMapExampleTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

!GMStaticMapExampleTile methodsFor: 'rendering' stamp: 'RichardStern 2/18/2015 08:18'!
renderDescriptionOn: html

	html 
		text: 'Support for a mercator projection allows to mimic a tiling behaviour with static maps - in this example, the specified map is shown together with its 8 adjacent tiles (For space reasons map size should be kept at a lower value). However, projection accuracy has some limits here ... so feel free to improve the algorithm ...'.

! !

!GMStaticMapExampleTile methodsFor: 'rendering' stamp: 'RichardStern 2/17/2015 10:10'!
renderMapOn: html
	"Override superclass - instead of single map, render 9 tile maps"

	self renderTileMapsOn: html.
! !

!GMStaticMapExampleTile methodsFor: 'rendering' stamp: 'RichardStern 2/17/2015 09:52'!
renderTileMapForDirection: aSymbol ofProjection: aProjection on: html

	| tileCenter |
	tileCenter := aProjection centerCoordinateOfAdjacentMapInDirection: aSymbol.
	
	html tableData
		with:
		[ 
			self renderTileMapWithCenter: tileCenter on: html.
		]! !

!GMStaticMapExampleTile methodsFor: 'rendering' stamp: 'RichardStern 2/17/2015 10:09'!
renderTileMapWithCenter: aGMStaticMapPoint on: html
	"Render a tile map with 1/3 of original size an given center. No markers etc considered here"
	
	^html googleStaticMap
		longitude: aGMStaticMapPoint longitude;
		latitude: aGMStaticMapPoint latitude;
		zoom: self zoom;
		width: (self width asNumber ) rounded;
		height: (self height asNumber ) rounded;
		mapType: self mapType asString;
		scale: self scale asNumber;
		language: self language;
		imageFormat: self imageFormat;
		yourself.
! !

!GMStaticMapExampleTile methodsFor: 'rendering' stamp: 'RichardStern 2/18/2015 08:19'!
renderTileMapsOn: html
	| proj |

	proj := self createProjection.
	html table 
		class: 'mapTiles';
		with:
		[ 		
			html tableRow with:
			[ 
				self 
					renderTileMapForDirection: #NorthWest ofProjection: proj on: html;
					renderTileMapForDirection: #North ofProjection: proj on: html;
					renderTileMapForDirection: #NorthEast ofProjection: proj on: html.
			].
			html tableRow with:
			[ 
				self 
					renderTileMapForDirection: #West ofProjection: proj on: html;
					renderTileMapForDirection: #None ofProjection: proj on: html;
					renderTileMapForDirection: #East ofProjection: proj on: html.
			].
			html tableRow with:
			[ 
				self 
					renderTileMapForDirection: #SouthWest ofProjection: proj on: html;
					renderTileMapForDirection: #South ofProjection: proj on: html;
					renderTileMapForDirection: #SouthEast ofProjection: proj on: html.
			].
		].
! !

!GMStaticMapExampleTile methodsFor: 'private' stamp: 'RichardStern 2/17/2015 10:13'!
setInitialParameters

	super setInitialParameters.
	self 
		width: 200;
		height: 200.! !

!GMStaticMapExampleTile methodsFor: 'rendering' stamp: 'RichardStern 2/12/2016 08:23'!
style

^super style,
'
table.mapTiles {border-collapse: separate; border-spacing: 1px;}
table.mapTiles td { margin: 0; padding: 0;}
table.mapTiles img { margin: 0; padding: 0;}
'.

		! !

WAComponent subclass: #GMStaticMapExampleBrowser
	instanceVariableNames: 'examples activeExample'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

!GMStaticMapExampleBrowser class methodsFor: 'examples' stamp: 'RichardStern 07/19/2015 04:28'!
example

	^self new.
! !

!GMStaticMapExampleBrowser methodsFor: 'accessing' stamp: 'rs 12/3/2013 13:19'!
activeExample
	^ activeExample! !

!GMStaticMapExampleBrowser methodsFor: 'accessing' stamp: 'rs 12/3/2013 13:19'!
activeExample: anObject
	activeExample := anObject! !

!GMStaticMapExampleBrowser methodsFor: 'private' stamp: 'rs 12/3/2013 13:38'!
children

	^Array with: self activeExample.! !

!GMStaticMapExampleBrowser methodsFor: 'accessing' stamp: 'rs 12/3/2013 13:19'!
examples
	^ examples! !

!GMStaticMapExampleBrowser methodsFor: 'accessing' stamp: 'rs 12/3/2013 13:19'!
examples: anObject
	examples := anObject! !

!GMStaticMapExampleBrowser methodsFor: 'private' stamp: 'RichardStern 2/17/2015 10:02'!
fillExamples

	self examples: OrderedCollection new.
	
	self examples
		add: 'Overall' -> (GMStaticMapOverallExample new);
		add: 'Map' -> (GMStaticMapExamplePlainMap new);
		add: 'Styled Map' -> (GMStaticMapExampleStyledMap new);
		add: 'Markers' -> (GMStaticMapExampleMarkers  new);
		add: 'Path' -> (GMStaticMapExamplePath new);
		add: 'Tile' -> (GMStaticMapExampleTile new);
		add: 'Move' -> (GMStaticMapExampleMove new);
		add: 'Address search' -> (GMStaticMapExampleAddressSearch new).
	! !

!GMStaticMapExampleBrowser methodsFor: 'initialization' stamp: 'rs 12/11/2013 08:08'!
initialize

	super initialize.
	self fillExamples.
	self activeExample: self examples first value.
	! !

!GMStaticMapExampleBrowser methodsFor: 'rendering' stamp: 'rs 12/3/2013 13:22'!
renderActiveExampleOn: html

	html div
		class: 'acitveExample';
		with: self activeExample.! !

!GMStaticMapExampleBrowser methodsFor: 'rendering' stamp: 'rs 12/3/2013 13:22'!
renderContentOn: html

	self
		renderSwitchesOn: html;
		renderActiveExampleOn: html.! !

!GMStaticMapExampleBrowser methodsFor: 'rendering' stamp: 'rs 12/11/2013 08:12'!
renderSwitchesOn: html

	html horizontalRule.

	self examples do:
	[:assoc |
		html anchor
			class: 'switch';
			class: 'active' if: (assoc value == self activeExample);
			callback: [self activeExample: assoc value];
			with: assoc key asString.
		html space: 2.
	].

	html horizontalRule; break.! !

!GMStaticMapExampleBrowser methodsFor: 'rendering' stamp: 'rs 12/11/2013 08:13'!
style

^'
a.switch {margin: 2px; padding: 4px; min-width: 2em; min-height: 1em; background-color: white; color: black; text-decoration: none;}
a.active {background-color: lightGray; font-weight: bold; border: 1px solid gray;}
'
! !

WAComponent subclass: #GMStaticMapOverallExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap-Examples'!

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'rs 12/3/2013 09:40'!
majorMarkersOnZeroMeridian

	| group |
	
	group := GMStaticMapStandardMarkerGroup new
							color: 'purple';
							size: 'small';
							yourself.			
		
	30 to: 60 by: 5 do:
	[:i | 
		group addPointWithLongitude: 0 andLatitude: i.
	].

	^group.! !

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'RichardStern 12/6/2010 13:51'!
mappointBetweenParisAndRome
	^GMStaticMapPoint new
			longitude: 11;
			latitude: 47;
			yourself.

	
! !

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'rs 3/29/2010 15:28'!
mappointBetweenViennaAndParis
	^GMStaticMapPoint new
			longitude: 7;
			latitude: 51;
			yourself.

	
! !

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'rs 12/3/2013 09:46'!
markerForParis
	^GMStaticMapStandardMarkerGroup  new
			addPointWithLongitude: 2.3 andLatitude: 48.8;	
			label: 'P';
			color: 'black';
			size: 'normal';
			yourself.

	
! !

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'rs 12/3/2013 09:37'!
markerForRome
	^GMStaticMapStandardMarkerGroup  new
			addPointWithLongitude: 12.50 andLatitude: 41.88;
			label: 'R';
			color: 'red';
			size: 'normal';
			yourself.

	
! !

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'RichardStern 1/3/2011 15:32'!
markerForVienna
	^GMStaticMapStandardMarkerGroup  new
			addPointWithLongitude: 16.3725 andLatitude: 48.2085;
			label: 'V';
			color: 'blue';
			size: 'normal';
			yourself.

	
! !

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'RichardStern 12/6/2010 14:48'!
markerGroups
	^OrderedCollection new
			add: self markerForVienna;
			add: self markerForParis;
			add: self markerForRome;
			add: self majorMarkersOnZeroMeridian;
			add: self minorMarkersOnZeroMeridian;			
			yourself.
	
! !

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'rs 12/3/2013 09:47'!
minorMarkersOnZeroMeridian

	| group |
	
	group := GMStaticMapStandardMarkerGroup  new
							color: 'yellow';
							size: 'tiny';
							yourself.

	31 to: 59 do:
	[:i | 
		group addPointWithLongitude: 0 andLatitude: i.
	].


	^group.! !

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'RichardStern 12/6/2010 14:45'!
path
	^GMStaticMapPath new
				color: 'red';
				fillcolor: 'brown';
				weight: 4;
				points: self pathPoints;
				yourself.! !

!GMStaticMapOverallExample methodsFor: 'private' stamp: 'RichardStern 12/6/2010 15:19'!
pathPoints
	^OrderedCollection new
			add: self markerForVienna firstPoint;
			add: self mappointBetweenViennaAndParis;
			add: self markerForParis firstPoint;
			add: self mappointBetweenParisAndRome;
			add: self markerForRome firstPoint;
			add: self markerForVienna firstPoint;
			yourself.
	
! !

!GMStaticMapOverallExample methodsFor: 'rendering' stamp: 'rs 12/3/2013 13:36'!
renderContentOn: html

	self renderDescriptionOn: html.

	html googleStaticMap
		longitude: 10;
		latitude: 48;
		zoom: 5;
		width: 500;
		height: 500;
		roadMap;
		addMarkerGroups: self markerGroups;
		path: self path.
		! !

!GMStaticMapOverallExample methodsFor: 'rendering' stamp: 'rs 12/3/2013 13:41'!
renderDescriptionOn: html

	html div
		class: 'description';
		with:
		[
		html 
			text: 'This example tries to demonstrate the use of most of the static map features at one glance. It displays';
			unorderedList:
			[	
				html 
					listItem: 'a road-style map on which';
					listItem: 'markers of different size are used to display the zero-meridian and';
					listItem: 'the path feature is used to overlay the map with a filled polygon'.
			].
		].
	
! !

!GMStaticMapOverallExample methodsFor: 'rendering' stamp: 'rs 12/3/2013 13:42'!
style

^'
div.description {float: left; width: 90%; margin-bottom: 1em; padding: 4px; background-color: lightYellow; border: 1px solid gray;}
'! !

Object subclass: #GMApiKey
	instanceVariableNames: 'apiKey'
	classVariableNames: 'Singleton'
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMApiKey commentStamp: '<historical>' prior: 0!
GMApiKey is a singleton to store the API key for Google maps. With version 2 of google static maps api, it is not required any more - but the class was kept for the case that a key is still used.

!

!GMApiKey class methodsFor: 'accessing' stamp: 'rs 12/26/2008 09:31'!
current
	"Return existing singleton instance. Create one if not yet present"
	
	Singleton isNil ifTrue: 
	[
		Singleton := self new
							apiKey: self defaultKey;
							yourself.
	].
	^Singleton.! !

!GMApiKey class methodsFor: 'accessing' stamp: 'rs 12/26/2008 09:30'!
defaultKey
	"Return google api key to use by default"
	
	^'Please enter your key here'.
! !

!GMApiKey class methodsFor: 'accessing' stamp: 'rs 12/26/2008 09:32'!
reset
	
	Singleton := nil.! !

!GMApiKey methodsFor: 'private' stamp: 'RichardStern 7/25/2010 09:53'!
addYourselfToUrl: aWAUrl
	"Add receiver as parameter with name 'key' to passed URL"
	
	aWAUrl 
		addField: 'key' 
		value: self apiKey.! !

!GMApiKey methodsFor: 'accessing' stamp: 'rs 12/23/2008 15:24'!
apiKey
	^ apiKey! !

!GMApiKey methodsFor: 'accessing' stamp: 'rs 12/23/2008 15:24'!
apiKey: anObject
	apiKey := anObject! !

Object subclass: #GMGeocodeRequest
	instanceVariableNames: 'address outputFormat cachedResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMGeocodeRequest commentStamp: '<historical>' prior: 0!
GMGeocodeRequest represents a single geocoding request against the static maps API. 

Instance Variables:
	address	<String>		The address fragment to search for
	outputFormat	<String> 	Desired format for result data - csv by default!

!GMGeocodeRequest class methodsFor: 'constants' stamp: 'rs 8/8/2013 08:16'!
supportedOutputFormats
	"Return a list of supported output formats for search result" 

	^#('json' 'xml')
! !

!GMGeocodeRequest methodsFor: 'accessing' stamp: 'rs 4/27/2009 13:20'!
address
	^ address! !

!GMGeocodeRequest methodsFor: 'accessing' stamp: 'rs 4/27/2009 13:22'!
address: anObject
	"set the address for whch to look up coordinates"
	address := anObject! !

!GMGeocodeRequest methodsFor: 'private' stamp: 'RichardStern 4/17/2014 08:18'!
assembleRequestUrl
	"Assemble URL of HTTP-Get-Request to GoogleMaps that receiver finally represents"
	"Now using Gecoding API V3 - see https://developers.google.com/maps/documentation/geocoding"

	^WAUrl new
		scheme: 'http';
		host: 'maps.googleapis.com';
		addToPath: 'maps';
		addToPath: 'api';
		addToPath: 'geocode';
		addToPath: self outputFormat asString;
		addField: 'address' value: self address;
		"Add the mandatory sensor parameter - use false by default (TODO: Make this configurable)"
		addField: 'sensor' value: 'false';
		asString.! !

!GMGeocodeRequest methodsFor: 'accessing' stamp: 'RichardStern 4/17/2014 11:02'!
cachedResult
	^ cachedResult! !

!GMGeocodeRequest methodsFor: 'accessing' stamp: 'RichardStern 4/17/2014 11:03'!
cachedResult: aString
	"Store result string of api call. Thus cached, it can be processed multiple times for 
	different purposes	(getting raw result, extracting markers, ...)"

	cachedResult := aString! !

!GMGeocodeRequest methodsFor: 'private' stamp: 'RichardStern 4/17/2014 11:05'!
execute
	"Execute the address lookup. Return a stream with the result (as got from Google) in the
	configured output format. Return avlue is furthermore cached in an instance variable for 
	multiple reuse"

	self 
		resetCachedResult;
"		cachedStream: (ZnHTTPSocketFacade httpGet: self assembleRequestUrl)."
		cachedResult: (ZnClient new
							beOneShot;
							url: self assembleRequestUrl;
							get) asString.
	self cachedResult.! !

!GMGeocodeRequest methodsFor: 'initializing' stamp: 'RichardStern 4/17/2014 08:26'!
initialize

	address := String new.
	outputFormat := self class supportedOutputFormats first asSymbol.! !

!GMGeocodeRequest methodsFor: 'accessing' stamp: 'rs 4/27/2009 13:21'!
outputFormat
	^ outputFormat! !

!GMGeocodeRequest methodsFor: 'accessing' stamp: 'RichardStern 12/15/2013 13:25'!
outputFormat: anObject
	"Set output format - allowed values are #xml and #json"
	outputFormat := anObject! !

!GMGeocodeRequest methodsFor: 'private' stamp: 'RichardStern 4/17/2014 11:04'!
resetCachedResult
	"Set result cache to nil"
	
	self cachedResult: nil.! !

!GMGeocodeRequest methodsFor: 'public' stamp: 'RichardStern 4/17/2014 11:21'!
resultMarkers
	"Create a marker for each of the result entries of the search
	(actually 34 markers at max that are named from 1 - 9 	and A - Z) and 
	return collection of these markers. "

	| dict markers |

	"Check if receiver was executed already for json format (as only parsing of json is 
	supported here). If not	execute the search "
	(self wasAlreadyExecuted and: [ self outputFormat == #json]) ifFalse:
	[ 
		self 
			outputFormat: #json;
			execute.
	].

	dict := JSJsonParser parse: self resultString.

	"Check for success - if not, return empty collection"
	((dict at: 'status' ifAbsent:['']) asUppercase = 'OK') ifFalse: [^#()].
	
	"Transform the result into marker instances"
	markers := OrderedCollection new.
	(dict at: 'results' ifAbsent: [#()]) doWithIndex:
	[:singleResultDict :i |
		(GMStaticMapStandardMarkerGroup newFromGeocodeResultEntry: singleResultDict index: i) 
		ifNotNilDo: 
		[ :m |
			markers add: m.
		].
	].

	^markers.! !

!GMGeocodeRequest methodsFor: 'public' stamp: 'RichardStern 4/17/2014 11:06'!
resultString
	"Return result string of the geocode request (as received from Google). Note that if 
	receiver was executed already, this will return a cached result"

	self wasAlreadyExecuted ifFalse: [ self execute. ].
	^self cachedResult.
	
! !

!GMGeocodeRequest methodsFor: 'private' stamp: 'RichardStern 4/17/2014 11:04'!
wasAlreadyExecuted
	"Return if receiver was already executed - which is given if there is a cached result"
	
	^self cachedResult isString.! !

Object subclass: #GMStaticMapMercatorProjection
	instanceVariableNames: 'zoom imageWidth imageHeight centerCoordinate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMStaticMapMercatorProjection methodsFor: 'public' stamp: 'RichardStern 12/2/2014 13:16'!
bottomCenterCoordinate

	^self coordinateOfPointWithCenterDelta: 
				(Point
				 	x: 0
					y: (self imageHeight / 2)).! !

!GMStaticMapMercatorProjection methodsFor: 'public' stamp: 'RichardStern 12/2/2014 10:25'!
bottomLeftCoordinate

	^self coordinateOfPointWithCenterDelta: 
				(Point
				 	x: (self imageWidth / 2) * -1
					y: (self imageHeight / 2)).! !

!GMStaticMapMercatorProjection methodsFor: 'public' stamp: 'RichardStern 12/2/2014 10:24'!
bottomRightCoordinate

	^self coordinateOfPointWithCenterDelta: 
				(Point
				 	x: (self imageWidth / 2) 
					y: (self imageHeight / 2)).! !

!GMStaticMapMercatorProjection methodsFor: 'accessing' stamp: 'RichardStern 12/1/2014 14:35'!
centerCoordinate
	^ centerCoordinate! !

!GMStaticMapMercatorProjection methodsFor: 'accessing' stamp: 'RichardStern 12/1/2014 14:35'!
centerCoordinate: anObject
	centerCoordinate := anObject! !

!GMStaticMapMercatorProjection methodsFor: 'public' stamp: 'RichardStern 2/17/2015 10:04'!
centerCoordinateOfAdjacentMapInDirection: aSymbol
	"Return the coordinate of the center of a map adjacent to the projcted one in the cardinal direction given as 
	symbol (e.g. #East, #SouthWest ,...). Special symbol #None will cause the unchanged center coordinates to be
	returned; any other value will cause an error"
	
	| signX signY  |

	(aSymbol == #None) ifTrue: [ ^self centerCoordinate. ].
	
	signX := 0.
	signY := 0.

	(aSymbol == #East) ifTrue: [ signX := 1.].
	(aSymbol == #West) ifTrue: [ signX := -1.].	
	(aSymbol == #North) ifTrue: [ signY := -1.].
	(aSymbol == #South) ifTrue: [ signY := 1.].	
	(aSymbol == #NorthEast) ifTrue: [ signX := 1. signY := -1.].
	(aSymbol == #SouthEast) ifTrue: [ signX := 1. signY := 1.].
	(aSymbol == #SouthWest) ifTrue: [ signX := -1. signY := 1.].
	(aSymbol == #NorthWest) ifTrue: [ signX := -1. signY := -1.].
	((signX == 0) and: [ signY == 0]) ifTrue: [^self error: 'Illegal cardinal direction parameter: ', aSymbol asString. ].
	
	^self coordinateOfPointWithCenterDelta:
			(Point 
					x: (self imageWidth * signX sign) 
					y: (self imageHeight * signY sign)).

! !

!GMStaticMapMercatorProjection methodsFor: 'public' stamp: 'RichardStern 12/2/2014 13:16'!
centerLeftCoordinate

	^self coordinateOfPointWithCenterDelta: 
				(Point
				 	x: (self imageWidth / 2) * -1
					y: 0).! !

!GMStaticMapMercatorProjection methodsFor: 'private-tiles' stamp: 'RichardStern 12/2/2014 10:44'!
centerPointOfTile
	"Return the center (Point) of a single tile"

 	^Point 	
		x: (self pixelsPerTile / 2)
		y: (self pixelsPerTile / 2).! !

!GMStaticMapMercatorProjection methodsFor: 'public' stamp: 'RichardStern 12/2/2014 13:17'!
centerRightCoordinate

	^self coordinateOfPointWithCenterDelta: 
				(Point
				 	x: (self imageWidth / 2) * -1
					y: 0).! !

!GMStaticMapMercatorProjection methodsFor: 'private-projection' stamp: 'RichardStern 9/21/2015 13:08'!
coordinateOfPoint: aPoint
	"Take passed point (pixels on map) and return corresponding coordinate (as GMStaticPoint). No scaling (zoom) considered here!!"
	
	| c long latRad lat |
	
	c := self centerPointOfTile.
	long := (aPoint x - c x) / self pixelsPerLongitudeDegree.
	latRad := (aPoint y - c y) / (self pixelsPerLongitudeRadian * -1).
	lat := ((2 * ((Float e raisedTo: latRad) arcTan)) - (Float pi /2)) radiansToDegrees.
	
	^GMStaticMapPoint new
			longitude: long asFloat;
			latitude: lat asFloat;
			yourself.
! !

!GMStaticMapMercatorProjection methodsFor: 'private-projection' stamp: 'RichardStern 12/2/2014 11:12'!
coordinateOfPointWithCenterDelta: aPoint
	"In map image, take point that is the given delta (pixels) away from image center. Return the coordinates of that point"

	| c p |

	c := self pointOfCoordinate: self centerCoordinate.
	p := Point 
			x: (c x + (aPoint x / self noOfTiles)) asFloat
			y: (c y + (aPoint y / self noOfTiles)) asFloat.
	^self coordinateOfPoint: p.
		
! !

!GMStaticMapMercatorProjection methodsFor: 'accessing' stamp: 'RichardStern 12/1/2014 13:27'!
imageHeight
	^ imageHeight! !

!GMStaticMapMercatorProjection methodsFor: 'accessing' stamp: 'RichardStern 12/1/2014 13:27'!
imageHeight: anObject
	imageHeight := anObject! !

!GMStaticMapMercatorProjection methodsFor: 'accessing' stamp: 'RichardStern 12/1/2014 13:27'!
imageWidth
	^ imageWidth! !

!GMStaticMapMercatorProjection methodsFor: 'accessing' stamp: 'RichardStern 12/1/2014 13:27'!
imageWidth: anObject
	imageWidth := anObject! !

!GMStaticMapMercatorProjection methodsFor: 'private-tiles' stamp: 'RichardStern 2/17/2015 10:56'!
noOfTiles
	"Return number of tiles to be used for a map of the whole world at given zoom"

	^2 raisedTo:  self zoom.

! !

!GMStaticMapMercatorProjection methodsFor: 'private-tiles' stamp: 'RichardStern 12/2/2014 10:46'!
pixelsPerLongitudeDegree
	"Return the no of pixel a single longitude degree needs on the world tile"

 	^(self pixelsPerTile / 360) asFloat.
! !

!GMStaticMapMercatorProjection methodsFor: 'private-tiles' stamp: 'RichardStern 12/2/2014 10:47'!
pixelsPerLongitudeRadian
	"Return the no of pixel a single longitude degree in radian format needs on the world tile"

 	^(self pixelsPerTile / (2 * Float pi)) asFloat.
! !

!GMStaticMapMercatorProjection methodsFor: 'private-tiles' stamp: 'RichardStern 2/17/2015 10:55'!
pixelsPerTile
	"Return the no of pixels used the top level tile (ie, the single tile of initial zoom level that contains whole 
	world). API uses 256x256 tiles at the moment"
	"Note: Only equal no of pixels on both axis supported at the moment"
	
	^256! !

!GMStaticMapMercatorProjection methodsFor: 'private-projection' stamp: 'RichardStern 12/2/2014 10:53'!
pointOfCoordinate: aGMStaticMapPoint
	"Take passed GM-Point (longitude/latitude) and return its point (pixel) within the map. No scaling (zoom) considered here!!"

	| c x y sinY|

      c := self centerPointOfTile.

	"Calculate x"
      x := (c x + (aGMStaticMapPoint longitude * self pixelsPerLongitudeDegree)).

	"Calculate y"
      "Note: Truncating to 0.9999 effectively limits latitude to # 89.189.  This is about a third of a tile past the edge of the world tile."
	sinY := aGMStaticMapPoint latitude degreesToRadians sin.
	sinY := (sinY max: -0.9999) min: 0.9999.	
	y := c y + ((0.5 * (((1 + sinY) / (1 - sinY)) ln )) * (self pixelsPerLongitudeRadian * -1)).
	
      ^Point 
		x: x rounded
		y: y rounded.! !

!GMStaticMapMercatorProjection methodsFor: 'public' stamp: 'RichardStern 12/2/2014 13:15'!
topCenterCoordinate

	^self coordinateOfPointWithCenterDelta: 
				(Point
				 	x: 0
					y: (self imageHeight / 2) * -1).! !

!GMStaticMapMercatorProjection methodsFor: 'public' stamp: 'RichardStern 12/2/2014 10:24'!
topLeftCoordinate

	^self coordinateOfPointWithCenterDelta: 
				(Point
				 	x: (self imageWidth / 2) * -1
					y: (self imageHeight / 2) * -1).! !

!GMStaticMapMercatorProjection methodsFor: 'public' stamp: 'RichardStern 12/2/2014 10:25'!
topRightCoordinate

	^self coordinateOfPointWithCenterDelta: 
				(Point
				 	x: (self imageWidth / 2) * -1
					y: (self imageHeight / 2)).! !

!GMStaticMapMercatorProjection methodsFor: 'accessing' stamp: 'RichardStern 12/1/2014 13:27'!
zoom
	^ zoom! !

!GMStaticMapMercatorProjection methodsFor: 'accessing' stamp: 'RichardStern 12/1/2014 13:27'!
zoom: anObject
	zoom := anObject! !

Object subclass: #GMStaticMapPoint
	instanceVariableNames: 'longitude latitude'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMStaticMapPoint commentStamp: '<historical>' prior: 0!
GMStaticMapPoint is a helper class that represents a single geocoordinate (longitude and latitude))

Instance Variables:
	longitude	<Float>
	llatitude: 	<Float>!

!GMStaticMapPoint methodsFor: 'private' stamp: 'rs 1/13/2010 09:19'!
asPointOnlyUrlParameterOn: aStream
	"Write only the URL-representation of receivers coordinate 
	information (and NOT anythink more) out on passed stream"
	
	aStream 
		nextPutAll: self latitude asString;
		nextPut: $,;
		nextPutAll: self longitude asString.! !

!GMStaticMapPoint methodsFor: 'private' stamp: 'rs 1/13/2010 09:18'!
asUrlParameterOn: aStream
	"Write out receivers representation in an URL-request to Google static 
	maps api on passed stream"
	
	^self asPointOnlyUrlParameterOn: aStream.! !

!GMStaticMapPoint methodsFor: 'public' stamp: 'rs 1/21/2010 15:28'!
displayString
	^self longitude asString,'/', self latitude asString.! !

!GMStaticMapPoint methodsFor: 'accessing' stamp: 'rs 1/12/2010 15:19'!
latitude
	^ latitude! !

!GMStaticMapPoint methodsFor: 'accessing' stamp: 'rs 1/12/2010 15:19'!
latitude: anObject
	latitude := anObject! !

!GMStaticMapPoint methodsFor: 'accessing' stamp: 'rs 1/12/2010 15:19'!
longitude
	^ longitude! !

!GMStaticMapPoint methodsFor: 'accessing' stamp: 'rs 1/12/2010 15:19'!
longitude: anObject
	longitude := anObject! !

Object subclass: #GMStaticMapPointGroup
	instanceVariableNames: 'points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMStaticMapPointGroup commentStamp: '<historical>' prior: 0!
GMStaticMapPointGroup is the base class for collections of map points. It especially takes care of correctly representing coordinates in Google API URLs. Thus, this is base functionality for any markers or paths shown on the map (Note that even a single marker is finally just a group with a single entry)

Instance Variables:
	points	<(SequenceableCollection of: GMStaticMapPoint)>!

GMStaticMapPointGroup subclass: #GMStaticMapIconMarkerGroup
	instanceVariableNames: 'icon shadow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMStaticMapIconMarkerGroup commentStamp: '<historical>' prior: 0!
GMStaticMapIconMarkerGroup represents a set of markers on a static map that are displayed by a custom icon instead of the standard marker icon.

Instance Variables:
	icon	<String>		Url of the icon
	shadow	<Boolean>		Indicates if icon should have shadow added
		

!

!GMStaticMapIconMarkerGroup methodsFor: 'private' stamp: 'RichardStern 2/12/2016 08:15'!
asUrlParameterOn: aStream
	"Write out receivers representation in an URL-request to Google static 
	maps api on passed stream"

	self hasIcon ifTrue: 
	[	aStream 
			nextPutAll: 'icon:';
			nextPutAll: self icon asString.
	].
	self hasShadow ifFalse: 
	[	aStream 
			nextPut: $|;
			nextPutAll: 'shadow:false'.
	].
		
	super asUrlParameterOn: aStream.			
! !

!GMStaticMapIconMarkerGroup methodsFor: 'private' stamp: 'RichardStern 1/3/2011 15:05'!
hasIcon
	^(self icon~~nil) and: [self icon isEmpty not]! !

!GMStaticMapIconMarkerGroup methodsFor: 'private' stamp: 'RichardStern 1/3/2011 15:05'!
hasShadow
	^(self shadow~~nil) and: [self shadow]! !

!GMStaticMapIconMarkerGroup methodsFor: 'accessing' stamp: 'RichardStern 1/3/2011 15:32'!
icon
	^ icon! !

!GMStaticMapIconMarkerGroup methodsFor: 'accessing' stamp: 'RichardStern 1/3/2011 15:32'!
icon: anObject
	icon := anObject! !

!GMStaticMapIconMarkerGroup methodsFor: 'initialize-release' stamp: 'RichardStern 1/3/2011 15:35'!
initialize
	super initialize.
	
	icon := String new.
	shadow := true.

	! !

!GMStaticMapIconMarkerGroup methodsFor: 'accessing' stamp: 'RichardStern 1/3/2011 15:33'!
shadow
	^ shadow! !

!GMStaticMapIconMarkerGroup methodsFor: 'accessing' stamp: 'RichardStern 2/12/2016 08:17'!
shadow: anObject
	shadow := anObject! !

GMStaticMapPointGroup subclass: #GMStaticMapPath
	instanceVariableNames: 'color fillcolor weight geodesic'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMStaticMapPath commentStamp: '<historical>' prior: 0!
GMStaticMapPath represents a set of coordinate points displayed on a map as a polyline (path)

Instance Variables:
	color		<String>		HTTP color vlaue - stroke color of the line
	weight		<Integer>		The width of the line (in pixels)!

!GMStaticMapPath methodsFor: 'private' stamp: 'RichardStern 2/11/2016 10:54'!
addYourselfToUrl: aWAUrl
	"Add receiver as parameter with name URL representing a Google-Maps-API-Call"

	self hasPoints ifTrue:
	[	| stream |	

		stream := WriteStream on: String new.	

		self hasColor ifTrue:
		[	stream 
				nextPutAll: 'color:';
				nextPutAll: self color asString;
				nextPut: $|.
		].
		self hasFillcolor ifTrue:
		[	stream 
				nextPutAll: 'fillcolor:';
				nextPutAll: self fillcolor asString;
				nextPut: $|.
		].
		self isGeodesic ifTrue:
		[	stream 
				nextPutAll: 'geodesic:true';
				nextPut: $|.
		].
		stream 
			nextPutAll: 'weight:';
			nextPutAll: self weightValue asString.

		super asUrlParameterOn: stream.
	
		aWAUrl addField: 'path' value: stream contents.
	].

! !

!GMStaticMapPath methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 11:43'!
color 	
	^color! !

!GMStaticMapPath methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 11:43'!
color: aString
	color := aString! !

!GMStaticMapPath methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 11:44'!
fillcolor
	^fillcolor! !

!GMStaticMapPath methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 11:44'!
fillcolor: aString
	fillcolor := aString! !

!GMStaticMapPath methodsFor: 'accessing' stamp: 'RichardStern 2/11/2016 10:50'!
geodesic
	^geodesic! !

!GMStaticMapPath methodsFor: 'accessing' stamp: 'RichardStern 2/11/2016 10:50'!
geodesic: aBoolean
	geodesic := aBoolean! !

!GMStaticMapPath methodsFor: 'private' stamp: 'RichardStern 12/6/2010 11:46'!
hasColor
	^self color isString and: [self color isEmpty not].! !

!GMStaticMapPath methodsFor: 'private' stamp: 'RichardStern 12/6/2010 11:46'!
hasFillcolor
	^self fillcolor isString and: [self fillcolor isEmpty not].! !

!GMStaticMapPath methodsFor: 'initialize - release' stamp: 'RichardStern 2/11/2016 10:53'!
initialize
	super initialize.
	
	geodesic := false.

	! !

!GMStaticMapPath methodsFor: 'private' stamp: 'RichardStern 2/11/2016 11:08'!
isGeodesic

	^self geodesic == true.! !

!GMStaticMapPath methodsFor: 'accessing' stamp: 'rs 1/12/2010 15:04'!
weight
	^ weight! !

!GMStaticMapPath methodsFor: 'accessing' stamp: 'rs 1/12/2010 15:04'!
weight: anObject
	weight := anObject! !

!GMStaticMapPath methodsFor: 'private' stamp: 'rs 1/12/2010 16:46'!
weightValue
	^self weight isNil 
		ifTrue: [1]
		ifFalse: [self weight].! !

!GMStaticMapPointGroup class methodsFor: 'constants' stamp: 'RichardStern 12/6/2010 15:20'!
supportedColors
	"Return predefined color names supported for map paths"
	
	^#('black' 'brown' 'green' 'purple' 'yellow' 'blue' 'gray' 'orange' 'red' 'white')! !

!GMStaticMapPointGroup methodsFor: 'public' stamp: 'RichardStern 12/6/2010 14:40'!
addPoint: aGMStaticMapPoint

	^self points add: aGMStaticMapPoint.! !

!GMStaticMapPointGroup methodsFor: 'public' stamp: 'RichardStern 12/6/2010 14:41'!
addPointWithLongitude: aLongitudeFloat andLatitude: aLatitudeFloat

	^self addPoint: (GMStaticMapPoint new
						longitude: aLongitudeFloat;
						latitude: aLatitudeFloat;
						yourself).! !

!GMStaticMapPointGroup methodsFor: 'private' stamp: 'RichardStern 12/6/2010 15:13'!
asUrlParameterOn: aStream
	"Write out receivers representation in an URL-request to Google static 
	maps api on passed stream"
		
	self points do:
	[:p |
		aStream nextPut: $|.			
		p asUrlParameterOn: aStream.	
	].
! !

!GMStaticMapPointGroup methodsFor: 'public' stamp: 'RichardStern 12/6/2010 14:59'!
firstPoint
	^self points first! !

!GMStaticMapPointGroup methodsFor: 'private' stamp: 'RichardStern 12/6/2010 15:13'!
hasPoints
	^(self points~~nil) and: [self points isEmpty not]! !

!GMStaticMapPointGroup methodsFor: 'initialize-release' stamp: 'RichardStern 12/6/2010 14:38'!
initialize
	super initialize.
	
	points := OrderedCollection new.

	! !

!GMStaticMapPointGroup methodsFor: 'private' stamp: 'RichardStern 12/6/2010 15:13'!
noOfPoints
	^self points size.! !

!GMStaticMapPointGroup methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 14:39'!
points
	^ points! !

!GMStaticMapPointGroup methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 14:39'!
points: anObject
	points := anObject! !

!GMStaticMapPointGroup methodsFor: 'private' stamp: 'RichardStern 12/6/2010 15:13'!
resetPoints
	self points: (self points class new).! !

GMStaticMapPointGroup subclass: #GMStaticMapStandardMarkerGroup
	instanceVariableNames: 'color label size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMStaticMapStandardMarkerGroup commentStamp: '<historical>' prior: 0!
GMStaticMapMarkerGroup represents a set of markers (coordinate points) on a static map that are displayed in equal fashion (same color, label and size)

Instance Variables:
	color	<String>	HTTP color vlaue - stroke color of the line
	label	<String>	Label of the marker (single character)
	size	<String>	String (enumeration) value declaring the size of the marker icon!

!GMStaticMapStandardMarkerGroup class methodsFor: 'instance creation' stamp: 'rs 8/9/2013 09:27'!
newFromGeocodeResultEntry: aDictionary index: anInteger
	"Assume passed dictionary to represent a dictionary parsed from one result entry 
	of JSON-response to a geocde request - and that the result entry's positions wihtin 
	the whole result is given by passed index. Now, create and return a marker 
	instance representing it - or nil if this is not possible. Passed index is used to
	derive some character/color used for marker"

	| colors marker long lat  |

	colors := self supportedColors.
	marker := nil.
	long := ((aDictionary at: 'geometry') at: 'location') at: 'lng'.
	lat := ((aDictionary at: 'geometry') at: 'location') at: 'lat'.
	
	(long isNumber and: [lat isNumber]) ifTrue:
	[	
		marker := self new.
		marker
			addPointWithLongitude: long andLatitude: lat;
			label: (Character digitValue: anInteger) asString;
			color: (colors at: (anInteger \\ colors size)+1)
	].
					
	^marker.! !

!GMStaticMapStandardMarkerGroup class methodsFor: 'constants' stamp: 'RichardStern 12/6/2010 13:19'!
supportedSizes
	"Return a list of markers sizes allowed for static maps" 

	^#('normal' 'mid' 'small' 'tiny')! !

!GMStaticMapStandardMarkerGroup methodsFor: 'private' stamp: 'RichardStern 1/3/2011 15:07'!
asUrlParameterOn: aStream
	"Write out receivers representation in an URL-request to Google static 
	maps api on passed stream"

	"Add size, color and character to URL (wth no delimiter!!)- note that latter 
	has to be lowercase - otherwhise default marker is used by google"
	self hasSize ifTrue: 
	[	aStream 
			nextPutAll: 'size:';
			nextPutAll: self size asString.
	].
	self hasColor ifTrue: 
	[	aStream 
			nextPut: $|;
			nextPutAll: 'color:';			
			nextPutAll: self color asString.
	].
	self hasLabel ifTrue: 
	[	aStream 
			nextPut: $|;
			nextPutAll: 'label:';						
			nextPut: self label first asUppercase.
	].
		
	super asUrlParameterOn: aStream.			
! !

!GMStaticMapStandardMarkerGroup methodsFor: 'accessing' stamp: 'rs 12/25/2008 15:55'!
color
	^ color! !

!GMStaticMapStandardMarkerGroup methodsFor: 'accessing' stamp: 'rs 12/25/2008 15:55'!
color: anObject
	color := anObject! !

!GMStaticMapStandardMarkerGroup methodsFor: 'private' stamp: 'rs 12/25/2008 16:12'!
hasColor
	^self color~~nil! !

!GMStaticMapStandardMarkerGroup methodsFor: 'private' stamp: 'RichardStern 12/6/2010 13:05'!
hasLabel

	^(self label isString) and: [self label isEmpty not]
	! !

!GMStaticMapStandardMarkerGroup methodsFor: 'private' stamp: 'rs 3/29/2010 16:11'!
hasSize
	^self size~~nil! !

!GMStaticMapStandardMarkerGroup methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 13:07'!
label
	^ label! !

!GMStaticMapStandardMarkerGroup methodsFor: 'accessing' stamp: 'RichardStern 12/6/2010 13:07'!
label: anObject
	label := anObject! !

!GMStaticMapStandardMarkerGroup methodsFor: 'accessing' stamp: 'rs 3/29/2010 16:10'!
size
	^ size! !

!GMStaticMapStandardMarkerGroup methodsFor: 'accessing' stamp: 'rs 3/29/2010 16:10'!
size: anObject
	size := anObject! !

Object subclass: #GMStaticMapStyle
	instanceVariableNames: 'feature element rules'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GoogleStaticMap'!

!GMStaticMapStyle commentStamp: '<historical>' prior: 0!
Helper class holding all the different  options that may be applied to a static map for custom styling. Besides simply attribute like color, lightness etc., it also holds multi-enum values used for map features and display rules.

Instance Variables
	element:		<Object>
	feature:		<Object>
	rules:		<Object>

element
	- xxxxx

feature
	- xxxxx

rules
	- xxxxx
!

!GMStaticMapStyle class methodsFor: 'constants' stamp: 'rs 12/3/2013 08:57'!
supportedElements
	"Return the list of element values the API supports"
		

	^#(
		'all'
		'geometry' 'geometry.fill' 'geometry.stroke' 
		'labels' 'labels.icon' 'labels.text' 'labels.text.fill' 'labels.text.stroke'
	)
 




! !

!GMStaticMapStyle class methodsFor: 'constants' stamp: 'rs 12/3/2013 08:57'!
supportedFeatures
	"Return the list of feature values the API supports"
		

	^#(
		'administrative' 'administrative.country' 'administrative.land' 'administrative.locality' 'administrative.neighborhood' 'administrative.province'
		'all'
		'landscape' 'landscape.man_made' 'landscape.natural' 'landscape.natural.landcover' 'andscape.natural.terrain'
		'poi' 'poi.attraction' 'poi.business' 'poi.government' 'poi.medical' 'poi.park' 'poi.place_of_worship' 'poi.school' 'poi.sports_complex'
		'road' 'road.arterial' 'road.highway' 'road.highway.controlled_access'  'road.local'
		'transit' 'transit.line' 'transit.station' 'transit.station.airport' 'transit.station.bus' 'transit.station.rail'
		'water'
	)
 
! !

!GMStaticMapStyle class methodsFor: 'constants' stamp: 'rs 12/11/2013 08:46'!
supportedStyles
	"Return the list of style rule keys the API supports"		

	^#('color' 'gamma' 'hue' 'inverse_lightness' 'lightness' 'saturation' 'visibility' 'weight')
 
! !

!GMStaticMapStyle class methodsFor: 'constants' stamp: 'rs 12/11/2013 10:55'!
supportedVisibilityValues
	"Return the list of visibility values the API supports"		

	^#('on' 'off' 'simplified').
 
! !

!GMStaticMapStyle methodsFor: 'private' stamp: 'rs 12/3/2013 08:59'!
addRule: anAssoc

	"Add passed association (assumed to have string key and value) to receivers rules. The rule is only added if the key
	is a supported rule name - otherwhise, the entry is ignored"
	
	(self class supportedStyles includes: anAssoc key) ifTrue:
	[
		self rules add: anAssoc.	
	]! !

!GMStaticMapStyle methodsFor: 'private' stamp: 'rs 12/11/2013 13:23'!
asUrlParameterOn: aStream
	"Write out receivers representation in an URL-request to Google static 
	maps api on passed stream"

	self hasFeature ifTrue: 
	[	aStream 
			nextPutAll: 'feature:';
			nextPutAll: self feature asString.
	].

	self hasElement ifTrue: 
	[	aStream 
			nextPut: $|;
			nextPutAll: 'element:';
			nextPutAll: self element asString.
	].

	self rules keysAndValuesDo:
	[:k :v |
		(v isString and: [v isEmpty not]) ifTrue:
		[
			aStream 
				nextPut: $|;
				nextPutAll: k asString;
				nextPut: $:;
				nextPutAll: v asString.
		].				
	].
	

! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:45'!
color
	^self rules at: 'color' ifAbsent: [nil].! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/3/2013 09:02'!
color: aValue

	self addRule: ('color' -> aValue).! !

!GMStaticMapStyle methodsFor: 'accessing' stamp: 'rs 11/8/2013 11:18'!
element
	^ element! !

!GMStaticMapStyle methodsFor: 'accessing' stamp: 'rs 11/8/2013 11:18'!
element: anObject
	element := anObject! !

!GMStaticMapStyle methodsFor: 'accessing' stamp: 'rs 11/8/2013 11:18'!
feature
	^ feature! !

!GMStaticMapStyle methodsFor: 'accessing' stamp: 'rs 11/8/2013 11:18'!
feature: anObject
	feature := anObject! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:45'!
gamma
	^self rules at: 'gamma' ifAbsent: [nil].! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/3/2013 09:02'!
gamma: aValue

	self addRule: ('gamma' -> aValue).! !

!GMStaticMapStyle methodsFor: 'private' stamp: 'rs 12/11/2013 12:24'!
hasElement
	^self element isString and: [self element isEmpty not].! !

!GMStaticMapStyle methodsFor: 'private' stamp: 'rs 11/8/2013 11:33'!
hasFeature 
	^self feature isString and: [self feature isEmpty not].! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:45'!
hue
	^self rules at: 'hue' ifAbsent: [nil].! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/3/2013 09:02'!
hue: aValue

	self addRule: ('hue' -> aValue).! !

!GMStaticMapStyle methodsFor: 'initialize-release' stamp: 'rs 11/8/2013 11:20'!
initialize 	

	super initialize.
	
	self resetRules.! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:46'!
inverseLightness
	^self rules at: 'inverseLightness' ifAbsent: [nil].! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:46'!
inverseLightness: aValue

	self addRule: ('inverse_lightness' -> aValue).! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:33'!
invertLightness: aValue

	self addRule: ('invert_lightness' -> aValue).! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:45'!
lightness
	^self rules at: 'lightness' ifAbsent: [nil].! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/3/2013 09:01'!
lightness: aValue

	self addRule: ('lightness' -> aValue).! !

!GMStaticMapStyle methodsFor: 'private' stamp: 'rs 11/8/2013 11:28'!
resetRules

	self rules: Dictionary new.! !

!GMStaticMapStyle methodsFor: 'accessing' stamp: 'rs 11/8/2013 11:19'!
rules
	^ rules! !

!GMStaticMapStyle methodsFor: 'accessing' stamp: 'rs 11/8/2013 11:19'!
rules: anObject
	rules := anObject! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:45'!
saturation
	^self rules at: 'saturation' ifAbsent: [nil].! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/3/2013 09:02'!
saturation: aValue

	self addRule: ('saturation' -> aValue).! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:46'!
visibility
	^self rules at: 'visibility' ifAbsent: [nil].! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/3/2013 09:02'!
visibility: aValue

	self addRule: ('visibility' -> aValue).! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/11/2013 08:46'!
weight
	^self rules at: 'weight' ifAbsent: [nil].! !

!GMStaticMapStyle methodsFor: 'accessing - rules facade' stamp: 'rs 12/3/2013 09:02'!
weight: aValue

	self addRule: ('weight' -> aValue).! !
